'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var require$$0 = _interopDefault(require('util'));
var events = _interopDefault(require('events'));
var string_decoder = _interopDefault(require('string_decoder'));
var stream = require('stream');
var stream__default = _interopDefault(stream);
var querystring = require('querystring');
var http = _interopDefault(require('http'));
var https = _interopDefault(require('https'));
var path = _interopDefault(require('path'));
var url = _interopDefault(require('url'));
var childProcess = _interopDefault(require('child_process'));
var fs = _interopDefault(require('fs'));

const always = x => () => x;
const noop = () => {};
const zipObject = keys => vals => keys.reduce((obj, val, index) => {
  obj[val] = vals[index];
  return obj;
}, {});
const times = fn => num => {
  const list = [];
  let idx = 0;
  while (idx < num) {
    list[idx] = fn(idx);
    idx += 1;
  }
  return list;
};

var isarray = Array.isArray || function (arr) {
  return Object.prototype.toString.call(arr) == '[object Array]';
};

var pathToRegexp_1 = pathToRegexp;
var parse_1 = parse;
var compile_1 = compile;
var tokensToFunction_1 = tokensToFunction;
var tokensToRegExp_1 = tokensToRegExp;
var PATH_REGEXP = new RegExp([
'(\\\\.)',
'([\\/.])?(?:(?:\\:(\\w+)(?:\\(((?:\\\\.|[^\\\\()])+)\\))?|\\(((?:\\\\.|[^\\\\()])+)\\))([+*?])?|(\\*))'].join('|'), 'g');
function parse(str, options) {
  var tokens = [];
  var key = 0;
  var index = 0;
  var path$$1 = '';
  var defaultDelimiter = options && options.delimiter || '/';
  var res;
  while ((res = PATH_REGEXP.exec(str)) != null) {
    var m = res[0];
    var escaped = res[1];
    var offset = res.index;
    path$$1 += str.slice(index, offset);
    index = offset + m.length;
    if (escaped) {
      path$$1 += escaped[1];
      continue;
    }
    var next = str[index];
    var prefix = res[2];
    var name = res[3];
    var capture = res[4];
    var group = res[5];
    var modifier = res[6];
    var asterisk = res[7];
    if (path$$1) {
      tokens.push(path$$1);
      path$$1 = '';
    }
    var partial = prefix != null && next != null && next !== prefix;
    var repeat = modifier === '+' || modifier === '*';
    var optional = modifier === '?' || modifier === '*';
    var delimiter = res[2] || defaultDelimiter;
    var pattern = capture || group;
    tokens.push({
      name: name || key++,
      prefix: prefix || '',
      delimiter: delimiter,
      optional: optional,
      repeat: repeat,
      partial: partial,
      asterisk: !!asterisk,
      pattern: pattern ? escapeGroup(pattern) : asterisk ? '.*' : '[^' + escapeString(delimiter) + ']+?'
    });
  }
  if (index < str.length) {
    path$$1 += str.substr(index);
  }
  if (path$$1) {
    tokens.push(path$$1);
  }
  return tokens;
}
function compile(str, options) {
  return tokensToFunction(parse(str, options));
}
function encodeURIComponentPretty(str) {
  return encodeURI(str).replace(/[\/?#]/g, function (c) {
    return '%' + c.charCodeAt(0).toString(16).toUpperCase();
  });
}
function encodeAsterisk(str) {
  return encodeURI(str).replace(/[?#]/g, function (c) {
    return '%' + c.charCodeAt(0).toString(16).toUpperCase();
  });
}
function tokensToFunction(tokens) {
  var matches = new Array(tokens.length);
  for (var i = 0; i < tokens.length; i++) {
    if (typeof tokens[i] === 'object') {
      matches[i] = new RegExp('^(?:' + tokens[i].pattern + ')$');
    }
  }
  return function (obj, opts) {
    var path$$1 = '';
    var data = obj || {};
    var options = opts || {};
    var encode = options.pretty ? encodeURIComponentPretty : encodeURIComponent;
    for (var i = 0; i < tokens.length; i++) {
      var token = tokens[i];
      if (typeof token === 'string') {
        path$$1 += token;
        continue;
      }
      var value = data[token.name];
      var segment;
      if (value == null) {
        if (token.optional) {
          if (token.partial) {
            path$$1 += token.prefix;
          }
          continue;
        } else {
          throw new TypeError('Expected "' + token.name + '" to be defined');
        }
      }
      if (isarray(value)) {
        if (!token.repeat) {
          throw new TypeError('Expected "' + token.name + '" to not repeat, but received `' + JSON.stringify(value) + '`');
        }
        if (value.length === 0) {
          if (token.optional) {
            continue;
          } else {
            throw new TypeError('Expected "' + token.name + '" to not be empty');
          }
        }
        for (var j = 0; j < value.length; j++) {
          segment = encode(value[j]);
          if (!matches[i].test(segment)) {
            throw new TypeError('Expected all "' + token.name + '" to match "' + token.pattern + '", but received `' + JSON.stringify(segment) + '`');
          }
          path$$1 += (j === 0 ? token.prefix : token.delimiter) + segment;
        }
        continue;
      }
      segment = token.asterisk ? encodeAsterisk(value) : encode(value);
      if (!matches[i].test(segment)) {
        throw new TypeError('Expected "' + token.name + '" to match "' + token.pattern + '", but received "' + segment + '"');
      }
      path$$1 += token.prefix + segment;
    }
    return path$$1;
  };
}
function escapeString(str) {
  return str.replace(/([.+*?=^!:${}()[\]|\/\\])/g, '\\$1');
}
function escapeGroup(group) {
  return group.replace(/([=!:$\/()])/g, '\\$1');
}
function attachKeys(re, keys) {
  re.keys = keys;
  return re;
}
function flags(options) {
  return options.sensitive ? '' : 'i';
}
function regexpToRegexp(path$$1, keys) {
  var groups = path$$1.source.match(/\((?!\?)/g);
  if (groups) {
    for (var i = 0; i < groups.length; i++) {
      keys.push({
        name: i,
        prefix: null,
        delimiter: null,
        optional: false,
        repeat: false,
        partial: false,
        asterisk: false,
        pattern: null
      });
    }
  }
  return attachKeys(path$$1, keys);
}
function arrayToRegexp(path$$1, keys, options) {
  var parts = [];
  for (var i = 0; i < path$$1.length; i++) {
    parts.push(pathToRegexp(path$$1[i], keys, options).source);
  }
  var regexp = new RegExp('(?:' + parts.join('|') + ')', flags(options));
  return attachKeys(regexp, keys);
}
function stringToRegexp(path$$1, keys, options) {
  return tokensToRegExp(parse(path$$1, options), keys, options);
}
function tokensToRegExp(tokens, keys, options) {
  if (!isarray(keys)) {
    options = keys || options;
    keys = [];
  }
  options = options || {};
  var strict = options.strict;
  var end = options.end !== false;
  var route = '';
  for (var i = 0; i < tokens.length; i++) {
    var token = tokens[i];
    if (typeof token === 'string') {
      route += escapeString(token);
    } else {
      var prefix = escapeString(token.prefix);
      var capture = '(?:' + token.pattern + ')';
      keys.push(token);
      if (token.repeat) {
        capture += '(?:' + prefix + capture + ')*';
      }
      if (token.optional) {
        if (!token.partial) {
          capture = '(?:' + prefix + '(' + capture + '))?';
        } else {
          capture = prefix + '(' + capture + ')?';
        }
      } else {
        capture = prefix + '(' + capture + ')';
      }
      route += capture;
    }
  }
  var delimiter = escapeString(options.delimiter || '/');
  var endsWithDelimiter = route.slice(-delimiter.length) === delimiter;
  if (!strict) {
    route = (endsWithDelimiter ? route.slice(0, -delimiter.length) : route) + '(?:' + delimiter + '(?=$))?';
  }
  if (end) {
    route += '$';
  } else {
    route += strict && endsWithDelimiter ? '' : '(?=' + delimiter + '|$)';
  }
  return attachKeys(new RegExp('^' + route, flags(options)), keys);
}
function pathToRegexp(path$$1, keys, options) {
  if (!isarray(keys)) {
    options = keys || options;
    keys = [];
  }
  options = options || {};
  if (path$$1 instanceof RegExp) {
    return regexpToRegexp(path$$1, keys);
  }
  if (isarray(path$$1)) {
    return arrayToRegexp( path$$1, keys, options);
  }
  return stringToRegexp( path$$1, keys, options);
}
pathToRegexp_1.parse = parse_1;
pathToRegexp_1.compile = compile_1;
pathToRegexp_1.tokensToFunction = tokensToFunction_1;
pathToRegexp_1.tokensToRegExp = tokensToRegExp_1;

function next(...args) {
  const pipeline = args.shift();
  if (pipeline.length > 0) {
    const [pipe, ...rest] = pipeline;
    const nextPipe = (...args) => {
      next(...[rest, ...args]);
    };
    pipe(...[...args, nextPipe]);
  }
}

const verbs = ['get', 'post', 'put', 'patch', 'delete'];
const verbsPlusAll = verbs.concat('all');
function getRouter() {
  const routes = [];
  const start = [];
  const end = [];
  const router = {
    route(path$$1) {
      let route = routes.filter(x => x.path === path$$1)[0];
      if (route) return route.pathRouter;
      const keys = [];
      const pathRouter = getAPathRouter();
      route = {
        path: path$$1,
        keys: keys,
        regexp: pathToRegexp_1(path$$1, keys),
        pathRouter: pathRouter
      };
      routes.push(route);
      return pathRouter;
    },
    go(path$$1, req, resp, cb) {
      const matched = routes.some(route => {
        let routeActions;
        const matches = route.regexp.exec(path$$1);
        if (!matches) return false;
        const verb = req.verb.toLowerCase();
        if (route.pathRouter.verbs[verb] != null) routeActions = route.pathRouter.verbs[verb];else if (route.pathRouter.verbs.all != null) routeActions = route.pathRouter.verbs.all;else return false;
        req = Object.assign({}, req, {
          params: keysToParams(route.keys, matches),
          matches: matches
        });
        let pipes = start.concat(routeActions).concat(end);
        if (typeof cb === 'function') pipes = pipes.concat(cb);
        next(pipes, req, resp);
        return true;
      });
      if (!matched) throw new Error(`Route: ${path$$1} does not match provided routes.`);
      function keysToParams(keys, matches) {
        return keys.reduce((params, key, index) => {
          params[key.name] = matches[index + 1];
          return params;
        }, {});
      }
    },
    reset() {
      routes.length = 0;
      start.length = 0;
      end.length = 0;
    },
    addStart(action) {
      start.push(action);
      return this;
    },
    addEnd(action) {
      end.push(action);
      return this;
    },
    verbs: verbs.reduce(function buildVerbObject(verbs, verb) {
      verbs[verb.toUpperCase()] = verb;
      return verbs;
    }, {})
  };
  verbsPlusAll.forEach(function addVerbToRouter(verb) {
    router[verb] = function addVerbToRouterInner(path$$1, action) {
      router.route(path$$1)[verb](action);
      return this;
    };
  });
  return router;
}
function getAPathRouter() {
  return verbsPlusAll.reduce((pathRouter, verb) => {
    pathRouter[verb] = function verbMapper(action) {
      pathRouter.verbs[verb] = pathRouter.verbs[verb] || [];
      pathRouter.verbs[verb].push(action);
      return pathRouter;
    };
    return pathRouter;
  }, { verbs: {} });
}

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

function Queue() {
    this._in = [];
    this._out = [];
    this.length = 0;
}
Queue.prototype.enqueueAll = function enqueueList(list) {
    this._in = this._in.concat(list);
    this.length += list.length;
};
Queue.prototype.enqueue = function enqueue(item) {
    this._in.push(item);
    this.length++;
};
Queue.prototype.dequeue = function dequeue() {
    if (this._out.length) {
        this.length--;
        return this._out.pop();
    } else if (this._in.length) {
        this._out = this._in.reverse();
        this._in = [];
        this.length--;
        return this._out.pop();
    } else {
        return undefined;
    }
};
Queue.prototype.clear = function clear() {
    this._out = [];
    this._in = [];
    this.length = 0;
};
Queue.prototype.toArray = function toArray() {
    var res = [];
    res = res.concat(this._out);
    res.reverse();
    res = res.concat(this._in);
    return res;
};
Queue.prototype.toString = function toString() {
    return this.toArray().toString();
};
var queue = Queue;

var intMap = createCommonjsModule(function (module) {
var hasOwn = Object.prototype.hasOwnProperty;
function IntMap() {
    this.map = {};
    this.size = 0;
}
IntMap.prototype.set = function set(key, value) {
    if (this.has(key)) {
        this.map[key] = value;
    } else {
        this.map[key] = value;
        this.size++;
    }
    return this;
};
IntMap.prototype.get = function get(key) {
    return this.map[key];
};
IntMap.prototype.delete = function intMapDelete(key) {
    var deleted = this.has(key);
    if (deleted) {
        delete this.map[key];
        this.size--;
    }
    return deleted;
};
IntMap.prototype.has = function has(key) {
    return hasOwn.call(this.map, key);
};
IntMap.prototype.forEach = function forEach(f, thisArg) {
    for (var key in this.map) {
        f.call(thisArg, this.map[key], key, this);
    }
};
if (commonjsGlobal.Map === void 0) {
    module.exports = IntMap;
} else {
    module.exports = commonjsGlobal.Map;
}
});

var lib = createCommonjsModule(function (module, exports) {
var inherits = require$$0.inherits;
var EventEmitter = events.EventEmitter;
var Decoder = string_decoder.StringDecoder;
var slice = Array.prototype.slice;
var hasOwn = Object.prototype.hasOwnProperty;
var _global = commonjsGlobal;
if (typeof commonjsGlobal !== 'undefined') {
    _global = commonjsGlobal;
} else if (typeof window !== 'undefined') {
    _global = window;
}
var _ = exports = module.exports = __(Stream);
function __(StreamCtor) {
    return function ( xs, secondArg, mappingHint) {
        var s = null;
        if (_.isUndefined(xs)) {
            s = new StreamCtor();
            s.writable = true;
        } else if (_.isStream(xs)) {
            if (!(xs instanceof StreamCtor)) {
                var ret = new StreamCtor();
                xs.on('error', ret.write.bind(ret));
                s = xs.pipe(ret);
            } else {
                s = xs;
            }
        } else if (_.isArray(xs)) {
            s = new StreamCtor();
            s._outgoing.enqueueAll(xs);
            s._outgoing.enqueue(_.nil);
        } else if (_.isFunction(xs)) {
            s = new StreamCtor(xs);
        } else if (_.isObject(xs)) {
            if (_.isFunction(xs.on) && _.isFunction(xs.pipe)) {
                var onFinish = _.isFunction(secondArg) ? secondArg : defaultReadableOnFinish;
                s = new StreamCtor();
                s.writable = true;
                pipeReadable(xs, onFinish, s);
                return s.map(function (x) {
                    return x;
                });
            } else if (_.isFunction(xs.then)) {
                s = promiseStream(StreamCtor, xs);
            }
            else if (_.isFunction(xs.next)) {
                    return iteratorStream(StreamCtor, xs);
                } else if (!_.isUndefined(_global.Symbol) && xs[_global.Symbol.iterator]) {
                    return iteratorStream(StreamCtor, xs[_global.Symbol.iterator]());
                } else {
                    throw new Error('Object was not a stream, promise, iterator or iterable: ' + typeof xs);
                }
        } else if (_.isString(xs)) {
            var mapper = hintMapper(mappingHint);
            s = new StreamCtor();
            var callback_func = function () {
                var ctx = mapper.apply(this, arguments);
                s.write(ctx);
            };
            secondArg.on(xs, callback_func);
            var removeMethod = secondArg.removeListener
            || secondArg.unbind;
            if (removeMethod) {
                s._destructors.push(function () {
                    removeMethod.call(secondArg, xs, callback_func);
                });
            }
        } else {
            throw new Error('Unexpected argument type to Stream constructor: ' + typeof xs);
        }
        return s;
    };
}
var isES5 = function () {
    return Function.prototype.bind && !this;
}();
function checkIsNumber(parameter, paramName) {
    if (typeof parameter != 'number') {
        throw new TypeError('Parameter "' + paramName + '" is not a number.');
    }
}
function checkRange(predicate, message) {
    if (!predicate) {
        throw new RangeError(message);
    }
}
_.isUndefined = function (x) {
    return typeof x === 'undefined';
};
_.isFunction = function (x) {
    return typeof x === 'function';
};
_.isObject = function (x) {
    return typeof x === 'object' && x !== null;
};
_.isString = function (x) {
    return typeof x === 'string';
};
_.isArray = Array.isArray || function (x) {
    return Object.prototype.toString.call(x) === '[object Array]';
};
if (typeof setImmediate === 'undefined') {
    _.setImmediate = function (fn) {
        setTimeout(fn, 0);
    };
} else {
    _.setImmediate = function (fn) {
        setImmediate(fn);
    };
}
if (!_global.nil) {
    _global.nil = {};
}
var nil = _.nil = _global.nil;
_.curry = function (fn ) {
    var args = slice.call(arguments);
    return _.ncurry.apply(this, [fn.length].concat(args));
};
_.ncurry = function (n, fn ) {
    var largs = slice.call(arguments, 2);
    if (largs.length >= n) {
        return fn.apply(this, largs.slice(0, n));
    }
    return _.partial.apply(this, [_.ncurry, n, fn].concat(largs));
};
_.partial = function (f ) {
    var args = slice.call(arguments, 1);
    return function () {
        return f.apply(this, args.concat(slice.call(arguments)));
    };
};
_.flip = _.curry(function (fn, x, y) {
    return fn(y, x);
});
_.compose = function () {
    var fns = slice.call(arguments).reverse();
    return _.seq.apply(null, fns);
};
_.seq = function () {
    var fns = slice.call(arguments);
    return function () {
        if (!fns.length) {
            return null;
        }
        var r = fns[0].apply(this, arguments);
        for (var i = 1; i < fns.length; i++) {
            r = fns[i].call(this, r);
        }
        return r;
    };
};
function nop() {
}
function defaultReadableOnFinish(readable, callback) {
    readable.once('error', callback);
    return function () {
        readable.removeListener('error', callback);
    };
}
function pipeReadable(xs, onFinish, stream$$1) {
    var response = onFinish(xs, streamEndCb);
    var unbound = false;
    var cleanup = null;
    var endOnError = true;
    if (_.isFunction(response)) {
        cleanup = response;
    } else if (response != null) {
        cleanup = response.onDestroy;
        endOnError = !response.continueOnError;
    }
    xs.pipe(stream$$1);
    stream$$1.onDestroy(unbind);
    function streamEndCb(error) {
        if (stream$$1._nil_pushed) {
            return;
        }
        if (error) {
            stream$$1.write(new StreamError(error));
        }
        if (error == null || endOnError) {
            unbind();
            stream$$1.end();
        }
    }
    function unbind() {
        if (unbound) {
            return;
        }
        unbound = true;
        if (cleanup) {
            cleanup();
        }
        if (xs.unpipe) {
            xs.unpipe(stream$$1);
        }
    }
}
function newPullFunction(xs) {
    return function pull(cb) {
        xs.pull(cb);
    };
}
function newDelegateGenerator(pull) {
    return function delegateGenerator(push, next) {
        var self = this;
        pull(function (err, x) {
            var old = self._defer_run_generator;
            self._defer_run_generator = true;
            push(err, x);
            if (x !== nil) {
                next();
            }
            self._defer_run_generator = old;
            if (!old && self._run_generator_deferred) {
                self._runGenerator();
            }
        });
    };
}
function promiseStream(StreamCtor, promise) {
    var nilScheduled = false;
    return new StreamCtor(function (push) {
        promise = promise.then(function (value) {
            nilScheduled = true;
            _.setImmediate(function () {
                push(null, value);
                push(null, nil);
            });
            return null;
        }, function (err) {
            nilScheduled = true;
            _.setImmediate(function () {
                push(err);
                push(null, nil);
            });
            return null;
        });
        if (_.isFunction(promise['finally'])) {
            promise['finally'](function () {
                if (!nilScheduled) {
                    _.setImmediate(function () {
                        push(null, nil);
                    });
                }
                return null;
            });
        }
    });
}
function iteratorStream(StreamCtor, it) {
    return new StreamCtor(function (push, next) {
        var iterElem, iterErr;
        try {
            iterElem = it.next();
        } catch (err) {
            iterErr = err;
        }
        if (iterErr) {
            push(iterErr);
            push(null, _.nil);
        } else if (iterElem.done) {
            if (!_.isUndefined(iterElem.value)) {
                push(null, iterElem.value);
            }
            push(null, _.nil);
        } else {
            push(null, iterElem.value);
            next();
        }
    });
}
function hintMapper(mappingHint) {
    var mappingHintType = typeof mappingHint;
    var mapper;
    if (mappingHintType === 'function') {
        mapper = mappingHint;
    } else if (mappingHintType === 'number') {
        mapper = function () {
            return slice.call(arguments, 0, mappingHint);
        };
    } else if (_.isArray(mappingHint)) {
        mapper = function () {
            var args = arguments;
            return mappingHint.reduce(function (ctx, hint, idx) {
                ctx[hint] = args[idx];
                return ctx;
            }, {});
        };
    } else {
        mapper = function (x) {
            return x;
        };
    }
    return mapper;
}
function pipeStream(src, dest, write, end, passAlongErrors) {
    var resume = null;
    var s = src.consume(function (err, x, push, next) {
        var canContinue;
        if (err) {
            if (passAlongErrors) {
                canContinue = write.call(dest, new StreamError(err));
            } else {
                src.emit('error', err);
                canContinue = true;
            }
        } else if (x === nil) {
            end.call(dest);
            return;
        } else {
            canContinue = write.call(dest, x);
        }
        if (canContinue !== false) {
            next();
        } else {
            resume = next;
        }
    });
    dest.on('drain', onConsumerDrain);
    src.onDestroy(function () {
        dest.removeListener('drain', onConsumerDrain);
    });
    dest.emit('pipe', src);
    s.resume();
    return dest;
    function onConsumerDrain() {
        if (resume) {
            resume();
            resume = null;
        }
    }
}
function Stream(generator) {
    EventEmitter.call(this);
    this.__HighlandStream__ = true;
    this.id = ('' + Math.random()).substr(2, 6);
    this.paused = true;
    this.ended = false;
    this.readable = true;
    this.writable = false;
    this._outgoing = new queue();
    this._observers = [];
    this._destructors = [];
    this._send_events = false;
    this._nil_pushed = false;
    this._explicitly_destroyed = false;
    this._request = null;
    this._multiplexer = null;
    this._consumer = null;
    this._generator = generator;
    this._generator_requested = true;
    this._defer_run_generator = false;
    this._run_generator_deferred = false;
    var self = this;
    this._push_fn = function (err, x) {
        if (x === nil) {
            self._run_generator_deferred = false;
        }
        self._writeOutgoing(err ? new StreamError(err) : x);
    };
    this._next_fn = function (xs) {
        if (self._explicitly_destroyed) {
            return;
        }
        if (self._nil_pushed) {
            throw new Error('Cannot call next after nil');
        }
        self._generator_requested = true;
        if (xs) {
            xs = self.create(xs);
            var pull = newPullFunction(xs);
            self._generator = newDelegateGenerator(pull);
        }
        if (!self.paused) {
            if (self._defer_run_generator) {
                self._run_generator_deferred = true;
            } else {
                _.setImmediate(function () {
                    self._runGenerator();
                });
            }
        }
    };
    this.on('newListener', function (ev) {
        if (ev === 'data') {
            self._send_events = true;
            _.setImmediate(self.resume.bind(self));
        } else if (ev === 'end') {
            self._send_events = true;
        }
    });
    this.on('removeListener', function (ev) {
        if (ev === 'end' || ev === 'data') {
            var end_listeners = self.listeners('end').length;
            var data_listeners = self.listeners('data').length;
            if (end_listeners + data_listeners === 0) {
                self._send_events = false;
            }
        }
    });
}
inherits(Stream, EventEmitter);
function _addMethod(proto, topLevel) {
    return function (name, f) {
        proto[name] = f;
        var n = f.length;
        function relevel(coerce) {
            return _.ncurry(n + 1, function () {
                var args = slice.call(arguments);
                var s = coerce(args.pop());
                return f.apply(s, args);
            });
        }
        topLevel[name] = relevel(topLevel);
        topLevel[name]._relevel = relevel;
    };
}
var addMethod = _addMethod(Stream.prototype, _);
function _addMethods(proto, topLevel, methods) {
    for (var p in methods) {
        if (hasOwn.call(methods, p)) {
            _addMethod(proto, topLevel)(p, methods[p]);
        }
    }
}
function _addToplevelMethod(topLevel) {
    return function (name, fn) {
        function relevel(_topLevel) {
            var bound = fn.bind(_topLevel);
            bound._relevel = relevel;
            return bound;
        }
        topLevel[name] = relevel(topLevel);
    };
}
var addToplevelMethod = _addToplevelMethod(_);
function _addToplevelMethods(topLevel, methods) {
    for (var p in methods) {
        if (hasOwn.call(methods, p)) {
            _addToplevelMethod(topLevel)(p, methods[p]);
        }
    }
}
function use(Super, originalTopLevel) {
    return function (methods, toplevelMethods) {
        function Sub() {
            Stream.apply(this, arguments);
        }
        inherits(Sub, Super);
        function topLevel() {
            return __(Sub).apply(null, arguments);
        }
        for (var p in originalTopLevel) {
            if (hasOwn.call(originalTopLevel, p)) {
                var fn = originalTopLevel[p];
                topLevel[p] = typeof fn._relevel === 'function' ? fn._relevel(topLevel) : fn;
            }
        }
        _addMethods(Sub.prototype, topLevel, methods || {});
        _addToplevelMethods(topLevel, toplevelMethods || {});
        topLevel.use = use(Sub, topLevel);
        return topLevel;
    };
}
_.use = use(Stream, _);
addToplevelMethod('of', function (x) {
    return _([x]);
});
addToplevelMethod('fromError', function (error) {
    return _(function (push) {
        push(error);
        push(null, _.nil);
    });
});
Stream.prototype.create = function () {
    return __(this.constructor).apply(null, arguments);
};
Stream.prototype.createChild = function createChild() {
    var child = this.create.apply(this, arguments);
    child.onDestroy(this.destroy.bind(this));
    return child;
};
function StreamError(err) {
    this.__HighlandStreamError__ = true;
    this.error = err;
}
_.isStream = function (x) {
    return _.isObject(x) && !!x.__HighlandStream__;
};
_._isStreamError = function (x) {
    return _.isObject(x) && !!x.__HighlandStreamError__;
};
_._isStreamRedirect = function (x) {
    return _.isObject(x) && !!x.__HighlandStreamRedirect__;
};
Stream.prototype._send = function (token) {
    var err = null,
        x;
    if (_._isStreamError(token)) {
        err = token.error;
        this.readable = false;
    } else {
        x = token;
    }
    if (x === nil) {
        this.readable = false;
    }
    if (this._request) {
        var cb = this._request;
        this._request = null;
        this.pause();
        cb(err, x);
    }
    var observers = this._observers;
    for (var j = 0, len2 = observers.length; j < len2; j++) {
        observers[j].write(token);
    }
    if (this._send_events) {
        if (err) {
            this.emit('error', err);
        } else if (x === nil) {
            this.emit('end');
        } else {
            this.emit('data', x);
        }
    }
    if (x === nil) {
        this._onEnd();
    }
};
Stream.prototype._onEnd = function _onEnd() {
    if (this.ended) {
        return;
    }
    this.pause();
    this.ended = true;
    var i, len, observer;
    for (i = 0, len = this._observers.length; i < len; i++) {
        observer = this._observers[i];
        if (observer.source === this) {
            observer.source = null;
        }
    }
    for (i = 0, len = this._destructors.length; i < len; i++) {
        this._destructors[i].call(this);
    }
    this._outgoing.clear();
    this._observers = [];
    this._destructors = [];
    this._request = null;
    this._multiplexer = null;
    this._consumer = null;
    this._generator = null;
    this._generator_requested = false;
    this._defer_run_generator = false;
    this._run_generator_deferred = false;
};
addMethod('pause', function () {
    this.paused = true;
});
Stream.prototype._emitNext = function (num) {
    var emitted = 0;
    while ((num == null || emitted < num) && this._outgoing.length && !this.paused) {
        this._send(this._outgoing.dequeue());
        emitted++;
    }
};
addMethod('resume', function () {
    if (!this.paused || this._consumer && this._consumer.paused || this._multiplexer && this._multiplexer.paused) {
        return;
    }
    this.paused = false;
    if (this._outgoing.length) {
        this._emitNext();
        if (this.paused) {
            return;
        }
    }
    if (this._generator) {
        this._runGenerator();
    } else {
        this.emit('drain');
    }
});
addMethod('end', function () {
    if (this._nil_pushed) {
        return;
    }
    this.write(nil);
});
addMethod('pipe', function (dest, options) {
    options = options || {};
    var canClose = dest !== process.stdout && dest !== process.stderr && options.end !== false;
    var end;
    if (canClose) {
        end = dest.end;
    } else {
        end = nop;
    }
    return pipeStream(this, dest, dest.write, end, false);
});
addMethod('destroy', function () {
    this._explicitly_destroyed = true;
    if (this.ended) {
        return;
    }
    this.readable = this.writable = false;
    if (!this._nil_pushed) {
        this.end();
    }
    this._onEnd();
});
Stream.prototype._writeOutgoing = function _writeOutgoing(token) {
    if (this._explicitly_destroyed) {
        return;
    }
    if (this._nil_pushed) {
        throw new Error('Cannot write to stream after nil');
    }
    if (token === _.nil) {
        this._nil_pushed = true;
    }
    if (token === nil || _._isStreamError(token)) {
        this.writable = false;
    }
    if (this.paused) {
        this._outgoing.enqueue(token);
    } else {
        this._send(token);
    }
};
Stream.prototype._runGenerator = function () {
    if (!this._generator_requested) {
        return;
    }
    if (this._defer_run_generator) {
        this._run_generator_deferred = true;
        return;
    }
    this._defer_run_generator = true;
    do {
        this._run_generator_deferred = false;
        this._generator_requested = false;
        var gen = this._generator;
        if (gen.__ConsumeGenerator__) {
            var source = gen.source;
            if (source._outgoing.length) {
                var token = source._outgoing.dequeue();
                source._request = gen.pullCb;
                source.paused = false;
                source._send(token);
            } else {
                source._request = gen.pullCb;
                source.resume();
            }
        } else {
            gen.call(this, this._push_fn, this._next_fn);
        }
    } while (this._run_generator_deferred);
    this._defer_run_generator = false;
};
Stream.prototype._removeObserver = function (s) {
    this._observers = this._observers.filter(function (o) {
        return o !== s;
    });
    if (s.source === this) {
        s.source = null;
    }
};
addMethod('consume', function (f) {
    var source = this,
        consumer;
    if (this._consumer) {
        throw new Error('Stream already being consumed, you must either fork() or observe()');
    }
    if (this._multiplexer) {
        throw new Error('Stream has been forked. You must either fork() or observe().');
    }
    var gen = function () {
        source.pull(pullCb);
    };
    gen.__ConsumeGenerator__ = true;
    gen.pullCb = pullCb;
    gen.source = source;
    consumer = this._consumer = this.createChild(gen);
    consumer.id = 'consume:' + consumer.id;
    return consumer;
    function pullCb(err, x) {
        if (consumer._defer_run_generator) {
            f(err, x, consumer._push_fn, consumer._next_fn);
        } else {
            consumer._defer_run_generator = true;
            f(err, x, consumer._push_fn, consumer._next_fn);
            consumer._defer_run_generator = false;
            if (consumer._run_generator_deferred) {
                consumer._runGenerator();
            }
        }
    }
});
addMethod('pull', function (f) {
    if (f == null) {
        throw new Error('Cannot pull() with a null callback.');
    }
    if (this._request) {
        f(new Error('Cannot service a second pull() request while one is in progress.'));
    }
    this._request = f;
    this.resume();
});
addMethod('write', function (x) {
    this._writeOutgoing(x);
    return !this.paused;
});
addMethod('fork', function () {
    if (this._requests) {
        throw new Error('Cannot fork a stream with an outstanding pull() request.');
    }
    if (this._consumer) {
        throw new Error('Cannot fork a stream that has already been consumed().');
    }
    if (!this._multiplexer) {
        this._multiplexer = new StreamMultiplexer(this);
    }
    var s = this._multiplexer.newStream();
    s.id = 'fork:' + s.id;
    return s;
});
addMethod('observe', function () {
    var s = this.create();
    s.id = 'observe:' + s.id;
    s.writable = false;
    s.onDestroy(this._removeObserver.bind(this, s));
    this._observers.push(s);
    return s;
});
addMethod('errors', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            f(err, push);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            push(null, x);
            next();
        }
    });
});
addMethod('stopOnError', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            f(err, push);
            push(null, nil);
        } else if (x === nil) {
            push(null, nil);
        } else {
            push(null, x);
            next();
        }
    });
});
addMethod('each', function (f) {
    var self = this;
    var s = this.consume(function (err, x, push, next) {
        if (err) {
            self.emit('error', err);
        } else if (x === nil) {
            push(null, nil);
        } else {
            f(x);
            next();
        }
    });
    s.resume();
    return s;
});
addMethod('apply', function (f) {
    return this.toArray(function (args) {
        f.apply(null, args);
    });
});
addMethod('toArray', function (f) {
    var self = this;
    return this.collect().pull(function (err, x) {
        if (err) {
            self.emit('error', err);
        } else {
            f(x);
        }
    });
});
addMethod('done', function (f) {
    if (this.ended) {
        f();
        return null;
    }
    var self = this;
    return this.consume(function (err, x, push, next) {
        if (err) {
            self.emit('error', err);
        } else if (x === nil) {
            f();
        } else {
            next();
        }
    }).resume();
});
Stream.prototype.onDestroy = function onDestroy(f) {
    this._destructors.push(f);
    return this;
};
addMethod('toCallback', function (cb) {
    var value;
    var hasValue = false;
    this.consume(function (err, x, push, next) {
        if (err) {
            push(null, nil);
            if (hasValue) {
                cb(new Error('toCallback called on stream emitting multiple values'));
            } else {
                cb(err);
            }
        } else if (x === nil) {
            if (hasValue) {
                cb(null, value);
            } else {
                cb();
            }
        } else {
            if (hasValue) {
                push(null, nil);
                cb(new Error('toCallback called on stream emitting multiple values'));
            } else {
                value = x;
                hasValue = true;
                next();
            }
        }
    }).resume();
});
addMethod('map', function (f) {
    if (!_.isFunction(f)) {
        throw new Error('map expects a function as its only argument.');
    }
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var fnVal, fnErr;
            try {
                fnVal = f(x);
            } catch (e) {
                fnErr = e;
            }
            push(fnErr, fnVal);
            next();
        }
    });
});
addMethod('doto', function (f) {
    return this.map(function (x) {
        f(x);
        return x;
    });
});
Stream.prototype.tap = Stream.prototype.doto;
_.tap = _.doto;
addMethod('ratelimit', function (num, ms) {
    if (num < 1) {
        throw new Error('Invalid number of operations per ms: ' + num);
    }
    var sent = 0;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            if (sent < num) {
                sent++;
                push(null, x);
                next();
            } else {
                setTimeout(function () {
                    sent = 1;
                    push(null, x);
                    next();
                }, ms);
            }
        }
    });
});
addMethod('flatMap', function (f) {
    return this.map(f).sequence();
});
addMethod('pluck', function (prop) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else if (_.isObject(x)) {
            push(null, x[prop]);
            next();
        } else {
            push(new Error('Expected Object, got ' + typeof x));
            next();
        }
    });
});
var objectOnly = _.curry(function (strategy, x) {
    if (_.isObject(x)) {
        return strategy(x);
    } else {
        throw new Error('Expected Object, got ' + typeof x);
    }
});
addMethod('pickBy', function (f) {
    return this.map(objectOnly(function (x) {
        var out = {};
        var seen = isES5 ? Object.create(null) : {};
        var obj = x;
        function testAndAdd(prop) {
            if (seen[prop] !== true && f(x[prop], prop)) {
                out[prop] = x[prop];
                seen[prop] = true;
            }
        }
        if (isES5) {
            do {
                Object.getOwnPropertyNames(obj).forEach(testAndAdd);
                obj = Object.getPrototypeOf(obj);
            } while (obj);
        } else {
            for (var k in x) {
                testAndAdd(k);
            }
        }
        return out;
    }));
});
addMethod('pick', function (properties) {
    return this.map(objectOnly(function (x) {
        var out = {};
        for (var i = 0, length = properties.length; i < length; i++) {
            var p = properties[i];
            if (p in x) {
                out[p] = x[p];
            }
        }
        return out;
    }));
});
addMethod('filter', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var fnVal, fnErr;
            try {
                fnVal = f(x);
            } catch (e) {
                fnErr = e;
            }
            if (fnErr) {
                push(fnErr);
            } else if (fnVal) {
                push(null, x);
            }
            next();
        }
    });
});
addMethod('flatFilter', function (f) {
    return this.flatMap(function (x) {
        return f(x).take(1).otherwise(errorStream()).flatMap(function (bool) {
            return _(bool ? [x] : []);
        });
    });
    function errorStream() {
        return _(function (push) {
            push(new Error('Stream returned by function was empty.'));
            push(null, _.nil);
        });
    }
});
addMethod('reject', function (f) {
    return this.filter(_.compose(_.not, f));
});
addMethod('find', function (f) {
    return this.filter(f).take(1);
});
addMethod('findWhere', function (props) {
    return this.where(props).take(1);
});
addMethod('group', function (f) {
    var lambda = _.isString(f) ? _.get(f) : f;
    return this.reduce(function (m, o) {
        var key = lambda(o);
        if (!hasOwn.call(m, key)) {
            m[key] = [];
        }
        m[key].push(o);
        return m;
    }, {});
});
addMethod('compact', function () {
    return this.filter(function (x) {
        return x;
    });
});
addMethod('where', function (props) {
    return this.filter(function (x) {
        for (var k in props) {
            if (x[k] !== props[k]) {
                return false;
            }
        }
        return true;
    });
});
addMethod('uniqBy', function (compare) {
    var uniques = [];
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var seen = false;
            var hasErr;
            for (var i = 0, len = uniques.length; i < len; i++) {
                try {
                    seen = compare(x, uniques[i]);
                } catch (e) {
                    hasErr = e;
                    seen = true;
                }
                if (seen) {
                    break;
                }
            }
            if (!seen) {
                uniques.push(x);
                push(null, x);
            }
            if (hasErr) {
                push(hasErr);
            }
            next();
        }
    });
});
addMethod('uniq', function () {
    if (!_.isUndefined(_global.Set)) {
        var uniques = new _global.Set(),
            size = uniques.size;
        return this.consume(function (err, x, push, next) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(err, x);
            }
            else if (x !== x) {
                    push(null, x);
                    next();
                } else {
                    uniques.add(x);
                    if (uniques.size > size) {
                        size = uniques.size;
                        push(null, x);
                    }
                    next();
                }
        });
    }
    return this.uniqBy(function (a, b) {
        return a === b;
    });
});
addMethod('zipAll', function () {
    var returned = 0;
    var z = [];
    var finished = false;
    function nextValue(index, max, src, push, next) {
        src.pull(function (err, x) {
            if (err) {
                push(err);
                nextValue(index, max, src, push, next);
            } else if (x === _.nil) {
                if (!finished) {
                    finished = true;
                    push(null, nil);
                }
            } else {
                returned++;
                z[index] = x;
                if (returned === max) {
                    push(null, z);
                    next();
                }
            }
        });
    }
    var self = this;
    return self.collect().flatMap(function (array) {
        if (!array.length) {
            return self.create([]);
        }
        return self.create(function (push, next) {
            returned = 0;
            z = [];
            for (var i = 0, length = array.length; i < length; i++) {
                nextValue(i, length, array[i], push, next);
            }
        });
    });
});
addMethod('zipEach', function (ys) {
    return this.create([this]).concat(this.create(ys).map(this.create.bind(this))).zipAll();
});
addMethod('zip', function (ys) {
    return this.create([this, this.create(ys)]).zipAll();
});
addMethod('batch', function (n) {
    return this.batchWithTimeOrCount(-1, n);
});
addMethod('batchWithTimeOrCount', function (ms, n) {
    var batched = [],
        timeout;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (batched.length > 0) {
                push(null, batched);
                clearTimeout(timeout);
            }
            push(null, nil);
        } else {
            batched.push(x);
            if (batched.length === n) {
                push(null, batched);
                batched = [];
                clearTimeout(timeout);
            } else if (batched.length === 1 && ms >= 0) {
                timeout = setTimeout(function () {
                    push(null, batched);
                    batched = [];
                }, ms);
            }
            next();
        }
    });
});
addMethod('intersperse', function (separator) {
    var started = false;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            if (started) {
                push(null, separator);
            } else {
                started = true;
            }
            push(null, x);
            next();
        }
    });
});
addMethod('splitBy', function (sep) {
    var decoder = new Decoder();
    var buffer = false;
    function drain(x, push) {
        buffer = (buffer || '') + decoder.write(x);
        var pieces = buffer.split(sep);
        buffer = pieces.pop();
        pieces.forEach(function (piece) {
            push(null, piece);
        });
    }
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (_.isString(buffer)) {
                drain(decoder.end(), push);
                push(null, buffer);
            }
            push(null, nil);
        } else {
            drain(x, push);
            next();
        }
    });
});
addMethod('split', function () {
    return this.splitBy(/\r?\n/);
});
addMethod('slice', function (start, end) {
    if (start == null) {
        start = 0;
    }
    if (end == null) {
        end = Infinity;
    }
    checkIsNumber(start, 'start');
    checkIsNumber(end, 'end');
    checkRange(start >= 0, 'start cannot be negative.');
    checkRange(end >= 0, 'end cannot be negative.');
    var index = 0;
    if (start === 0 && end === Infinity) {
        return this;
    } else if (start >= end) {
        return _([]);
    }
    var s = this.consume(function (err, x, push, next) {
        var done = x === nil;
        if (err) {
            push(err);
        } else if (!done && index++ >= start) {
            push(null, x);
        }
        if (!done && index < end) {
            next();
        } else {
            push(null, nil);
        }
    });
    s.id = 'slice:' + s.id;
    return s;
});
addMethod('take', function (n) {
    var s = this.slice(0, n);
    s.id = 'take:' + s.id;
    return s;
});
addMethod('drop', function (n) {
    return this.slice(n, Infinity);
});
addMethod('head', function () {
    return this.take(1);
});
addMethod('last', function () {
    var nothing = {};
    var prev = nothing;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (prev !== nothing) {
                push(null, prev);
            }
            push(null, nil);
        } else {
            prev = x;
            next();
        }
    });
});
addMethod('sortBy', function (f) {
    return this.collect().invoke('sort', [f]).sequence();
});
addMethod('sort', function () {
    return this.sortBy();
});
addMethod('through', function (target) {
    var output;
    if (_.isFunction(target)) {
        return target(this);
    } else {
        target.pause();
        output = this.createChild();
        this.on('error', writeErr);
        target.on('error', writeErr);
        return this.pipe(target).pipe(output);
    }
    function writeErr(err) {
        output.write(new StreamError(err));
    }
});
addToplevelMethod('pipeline', function () {
    if (!arguments.length) {
        return this();
    }
    var start = arguments[0],
        rest,
        startHighland;
    if (!_.isStream(start) && !_.isFunction(start.resume)) {
        start = this();
        startHighland = start;
        rest = slice.call(arguments);
    } else {
        startHighland = this(start);
        rest = slice.call(arguments, 1);
    }
    var end = rest.reduce(function (src, dest) {
        return src.through(dest);
    }, startHighland);
    var wrapper = this(function (push, next) {
        end.pull(function (err, x) {
            push(err, x);
            if (x !== nil) {
                next();
            }
        });
    });
    wrapper.writable = true;
    wrapper.write = function (x) {
        return start.write(x);
    };
    wrapper.end = function () {
        wrapper.writable = false;
        return start.end();
    };
    start.on('drain', function () {
        wrapper.emit('drain');
    });
    return wrapper;
});
addMethod('sequence', function () {
    var original = this;
    var curr = this;
    return this.create(function (push, next) {
        curr.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (_.isArray(x)) {
                if (onOriginalStream()) {
                    x.forEach(function (y) {
                        push(null, y);
                    });
                } else {
                    push(null, x);
                }
                next();
            } else if (_.isStream(x)) {
                if (onOriginalStream()) {
                    curr = x;
                    next();
                } else {
                    push(null, x);
                    next();
                }
            } else if (x === nil) {
                if (onOriginalStream()) {
                    push(null, nil);
                } else {
                    curr = original;
                    next();
                }
            } else {
                if (onOriginalStream()) {
                    push(new Error('Expected Stream, got ' + typeof x));
                    next();
                } else {
                    push(null, x);
                    next();
                }
            }
        });
    });
    function onOriginalStream() {
        return curr === original;
    }
});
Stream.prototype.series = Stream.prototype.sequence;
_.series = _.sequence;
addMethod('flatten', function () {
    var curr = this;
    var stack = [];
    return this.createChild(function (push, next) {
        curr.pull(function (err, x) {
            if (err) {
                push(err);
                next();
                return;
            }
            if (_.isArray(x)) {
                x = _(x);
            }
            if (_.isStream(x)) {
                stack.push(curr);
                curr = x;
                next();
            } else if (x === nil) {
                if (stack.length) {
                    curr = stack.pop();
                    next();
                } else {
                    push(null, nil);
                }
            } else {
                push(null, x);
                next();
            }
        });
    });
});
addMethod('parallel', function (n) {
    var source = this;
    var running = [];
    var ended = false;
    var reading_source = false;
    if (typeof n !== 'number') {
        throw new Error('Must specify a number to parallel().');
    }
    if (n <= 0) {
        throw new Error('The parallelism factor must be positive');
    }
    return this.create(function (push, next) {
        if (running.length < n && !ended && !reading_source) {
            reading_source = true;
            source.pull(function (err, x) {
                reading_source = false;
                if (err) {
                    push(err);
                } else if (x === nil) {
                    ended = true;
                } else if (!_.isStream(x)) {
                    push(new Error('Expected Stream, got ' + typeof x));
                } else {
                    var run = { stream: x, buffer: [] };
                    running.push(run);
                    x.consume(function (_err, y, _push, _next) {
                        if (running[0] === run) {
                            if (y === nil) {
                                running.shift();
                                flushBuffer();
                                next();
                            } else {
                                push(_err, y);
                            }
                        } else {
                            run.buffer.push([_err, y]);
                        }
                        if (y !== nil) {
                            _next();
                        }
                    }).resume();
                }
                return next();
            });
        } else if (!running.length && ended) {
            push(null, nil);
        }
        function flushBuffer() {
            while (running.length && running[0].buffer.length) {
                var buf = running[0].buffer;
                for (var i = 0; i < buf.length; i++) {
                    if (buf[i][1] === nil) {
                        running.shift();
                        break;
                    } else {
                        push.apply(null, buf[i]);
                    }
                }
                buf.length = 0;
            }
        }
    });
});
addMethod('otherwise', function (ys) {
    var xs = this;
    return xs.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (_.isFunction(ys)) {
                next(ys());
            } else {
                next(ys);
            }
        } else {
            push(null, x);
            next(xs);
        }
    });
});
addMethod('append', function (y) {
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, y);
            push(null, _.nil);
        } else {
            push(err, x);
            next();
        }
    });
});
addMethod('reduce', function (f, z) {
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, z);
            push(null, _.nil);
        } else if (err) {
            push(err);
            next();
        } else {
            try {
                z = f(z, x);
            } catch (e) {
                push(e);
                push(null, _.nil);
                return;
            }
            next();
        }
    });
});
addMethod('reduce1', function (f) {
    var self = this;
    return self.createChild(function (push, next) {
        self.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                next(self.reduce(f, x));
            }
        });
    });
});
addMethod('collect', function () {
    var xs = [];
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, xs);
            push(null, nil);
        } else {
            xs.push(x);
            next();
        }
    });
});
addMethod('scan', function (f, z) {
    var self = this;
    return this.create([z]).concat(self.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, _.nil);
        } else if (err) {
            push(err);
            next();
        } else {
            try {
                z = f(z, x);
            } catch (e) {
                push(e);
                push(null, _.nil);
                return;
            }
            push(null, z);
            next();
        }
    }));
});
addMethod('scan1', function (f) {
    var self = this;
    return self.createChild(function (push, next) {
        self.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                next(self.scan(f, x));
            }
        });
    });
});
function HighlandTransform(push) {
    this.push = push;
}
HighlandTransform.prototype['@@transducer/init'] = function () {
    return this.push;
};
HighlandTransform.prototype['@@transducer/result'] = function (push) {
    return push;
};
HighlandTransform.prototype['@@transducer/step'] = function (push, input) {
    push(null, input);
    return push;
};
addMethod('transduce', function transduce(xf) {
    var transform = null,
        memo = null;
    return this.consume(function (err, x, push, next) {
        if (transform == null) {
            transform = xf(new HighlandTransform(push));
            memo = transform['@@transducer/init']();
        }
        if (err) {
            push(err);
            next();
        } else if (x === _.nil) {
            runResult(push, memo);
        } else {
            var res = runStep(push, memo, x);
            if (!res) {
                return;
            }
            memo = res;
            if (memo['@@transducer/reduced']) {
                runResult(memo['@@transducer/value']);
            } else {
                next();
            }
        }
    });
    function runResult(push, _memo) {
        try {
            transform['@@transducer/result'](_memo);
        } catch (e) {
            push(e);
        }
        push(null, _.nil);
    }
    function runStep(push, _memo, x) {
        try {
            return transform['@@transducer/step'](_memo, x);
        } catch (e) {
            push(e);
            push(null, _.nil);
            return null;
        }
    }
});
addMethod('concat', function (ys) {
    ys = _(ys);
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            next(ys);
        } else {
            push(err, x);
            next();
        }
    });
});
addMethod('merge', function () {
    var self = this;
    var srcs = [];
    var srcsNeedPull = [],
        first = true,
        async = false;
    return this.create(function (push, next) {
        if (first) {
            first = false;
            getSourcesSync(push, next);
        }
        if (srcs.length === 0) {
            push(null, nil);
        } else if (srcsNeedPull.length) {
            pullFromAllSources(push, next);
            next();
        } else {
            async = true;
        }
    });
    function srcPullHandler(push, next, src) {
        return function (err, x) {
            if (err) {
                push(err);
                srcsNeedPull.push(src);
            } else if (x === nil) {
                srcs = srcs.filter(function (s) {
                    return s !== src;
                });
            } else {
                if (src === self) {
                    srcs.push(x);
                    srcsNeedPull.push(x);
                    srcsNeedPull.unshift(self);
                } else {
                    push(null, x);
                    srcsNeedPull.push(src);
                }
            }
            if (async) {
                async = false;
                next();
            }
        };
    }
    function pullFromAllSources(push, next) {
        var _srcs = srcsNeedPull;
        srcsNeedPull = [];
        _srcs.forEach(function (src) {
            src.pull(srcPullHandler(push, next, src));
        });
    }
    function getSourcesSync(push, next) {
        var asynchronous;
        var done = false;
        var pull_cb = function (err, x) {
            asynchronous = false;
            if (done) {
                srcPullHandler(push, next, self)(err, x);
            } else {
                if (err) {
                    push(err);
                } else if (x === nil) {
                    done = true;
                } else {
                    srcs.push(x);
                    srcsNeedPull.push(x);
                }
            }
        };
        while (!done) {
            asynchronous = true;
            self.pull(pull_cb);
            if (asynchronous) {
                done = true;
                srcs.unshift(self);
            }
        }
    }
});
addMethod('mergeWithLimit', function mergeWithLimit(n) {
    var self = this;
    var processCount = 0;
    var waiting = false;
    if (typeof n !== 'number' || n < 1) {
        throw new Error('mergeWithLimit expects a positive number, but got: ' + n);
    }
    if (n === Infinity) {
        return this.merge();
    }
    return this.create(function (push, next) {
        self.pull(function (err, x) {
            var done = x === nil;
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                processCount++;
                push(err, x);
                x.onDestroy(function () {
                    processCount--;
                    if (waiting) {
                        waiting = false;
                        next();
                    }
                });
                if (!done && processCount < n) {
                    next();
                } else {
                    waiting = true;
                }
            }
        });
    }).merge();
});
addMethod('invoke', function (method, args) {
    return this.map(function (x) {
        return x[method].apply(x, args);
    });
});
addMethod('nfcall', function (args) {
    var wrap = _.wrapCallback._relevel(this.create.bind(this));
    return this.map(function (x) {
        return wrap(x).apply(x, args);
    });
});
addMethod('throttle', function (ms) {
    var last = 0 - ms;
    return this.consume(function (err, x, push, next) {
        var now = new Date().getTime();
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else if (now - ms >= last) {
            last = now;
            push(null, x);
            next();
        } else {
            next();
        }
    });
});
addMethod('debounce', function (ms) {
    var t = null;
    var nothing = {};
    var last = nothing;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (t) {
                clearTimeout(t);
            }
            if (last !== nothing) {
                push(null, last);
            }
            push(null, nil);
        } else {
            last = x;
            if (t) {
                clearTimeout(t);
            }
            t = setTimeout(push.bind(this, null, x), ms);
            next();
        }
    });
});
addMethod('latest', function () {
    var nothing = {},
        latest = nothing,
        errors = [],
        ended = false,
        onValue = null;
    this.consume(function (err, x, push, next) {
        if (onValue != null) {
            var cb = onValue;
            onValue = null;
            cb(err, x);
        }
        if (err) {
            errors.push(err);
            next();
        } else if (x === nil) {
            ended = true;
        } else {
            latest = x;
            next();
        }
    }).resume();
    return this.create(function (push, next) {
        var oldErrors = errors;
        errors = [];
        if (!oldErrors.length && latest === nothing && !ended) {
            onValue = function (err, x) {
                push(err, x);
                if (x !== nil) {
                    next();
                }
            };
        } else {
            oldErrors.forEach(push);
            if (latest !== nothing) {
                push(null, latest);
            }
            if (ended) {
                push(null, nil);
            } else {
                next();
            }
        }
    });
});
function StreamMultiplexer(stream$$1) {
    this._stream = stream$$1;
    this._consumers = new intMap();
    this._requests = new intMap();
    this._nonce = 0;
    this._emitting = false;
    this._repeatEmit = false;
    this._cached_value = null;
    this.paused = true;
    var self = this;
    this._pullCb = function _pullCb(err, x) {
        self.paused = true;
        if (self._requests.size === self._consumers.size) {
            self._send(err, x);
        } else {
            self._saved_token = [err, x];
        }
    };
}
StreamMultiplexer.prototype._send = function _send(err, x) {
    var requests = this._requests;
    this._requests = new intMap();
    requests.forEach(function (requestCb) {
        requestCb(err, x);
    });
};
StreamMultiplexer.prototype._resume = function _resume() {
    if (this._emitting) {
        this._repeatEmit = true;
        return;
    }
    this._emitting = true;
    do {
        this._repeatEmit = false;
        if (this._requests.size === this._consumers.size) {
            if (this._saved_token) {
                var token = this._saved_token;
                this._saved_token = null;
                this.paused = true;
                this._send(token[0], token[1]);
            } else if (this.paused) {
                this.paused = false;
                this._stream.pull(this._pullCb);
            }
        }
    } while (this._repeatEmit);
    this._emitting = false;
};
StreamMultiplexer.prototype.pull = function pull(id, cb) {
    if (!cb) {
        return;
    }
    if (this._requests.has(id)) {
        cb(new Error('Cannot service a second pull() request while one is in progress.'));
        return;
    }
    this._requests.set(id, cb);
    this._resume();
};
StreamMultiplexer.prototype.newStream = function newStream() {
    var self = this,
        id = self._nonce++;
    self._consumers.set(id, true);
    var pull = this.pull.bind(this, id);
    return new this._stream.constructor(newDelegateGenerator(pull)).onDestroy(function () {
        self.removeConsumer(id);
    });
};
StreamMultiplexer.prototype.removeConsumer = function removeConsumer(id) {
    if (!this._consumers.has(id)) {
        return;
    }
    this._consumers.delete(id);
    this._requests.delete(id);
    this._resume();
};
_.values = function (obj) {
    return _.keys(obj).map(function (k) {
        return obj[k];
    });
};
function keys(obj) {
    var keysArray = [];
    for (var k in obj) {
        if (hasOwn.call(obj, k)) {
            keysArray.push(k);
        }
    }
    return keysArray;
}
_.keys = function (obj) {
    return _(keys(obj));
};
_.pairs = function (obj) {
    return _.keys(obj).map(function (k) {
        return [k, obj[k]];
    });
};
_.extend = _.curry(function (extensions, target) {
    for (var k in extensions) {
        if (hasOwn.call(extensions, k)) {
            target[k] = extensions[k];
        }
    }
    return target;
});
_.get = _.curry(function (prop, obj) {
    return obj[prop];
});
_.set = _.curry(function (prop, val, obj) {
    obj[prop] = val;
    return obj;
});
_.log = function () {
    console.log.apply(console, arguments);
};
addToplevelMethod('wrapCallback', function (f, mappingHint) {
    var stream$$1 = this;
    var mapper = hintMapper(mappingHint);
    return function () {
        var self = this;
        var args = slice.call(arguments);
        return stream$$1(function (push) {
            var cb = function (err) {
                if (err) {
                    push(err);
                } else {
                    var cbArgs = slice.call(arguments, 1);
                    var v = mapper.apply(this, cbArgs);
                    push(null, v);
                }
                push(null, nil);
            };
            f.apply(self, args.concat([cb]));
        });
    };
});
addToplevelMethod('wrapAsync', function (f) {
    var stream$$1 = this;
    return function () {
        var promise;
        try {
            promise = f.apply(this, arguments);
            if (!_.isObject(promise) || !_.isFunction(promise.then)) {
                return _.fromError(new Error('Wrapped function did not return a promise'));
            }
            return stream$$1(promise);
        } catch (e) {
            return _.fromError(e);
        }
    };
});
function isClass(fn) {
    if (!(typeof fn === 'function' && fn.prototype)) {
        return false;
    }
    var getKeys = isES5 ? Object.getOwnPropertyNames : keys;
    var allKeys = getKeys(fn.prototype);
    return allKeys.length > 0 && !(allKeys.length === 1 && allKeys[0] === 'constructor');
}
function inheritedKeys(obj) {
    var allProps = {};
    var curr = obj;
    var handleProp = function (prop) {
        allProps[prop] = true;
    };
    while (Object.getPrototypeOf(curr)) {
        var props = Object.getOwnPropertyNames(curr);
        props.forEach(handleProp);
        curr = Object.getPrototypeOf(curr);
    }
    return keys(allProps);
}
function streamifyAll(wrapCallback, inp, suffix) {
    var getKeys = isES5 ? inheritedKeys : keys;
    var allKeys = getKeys(inp);
    for (var i = 0, len = allKeys.length; i < len; i++) {
        var key = allKeys[i];
        var val;
        try {
            val = inp[key];
        } catch (e) {
        }
        if (val && typeof val === 'function' && !isClass(val) && !val.__HighlandStreamifiedFunction__) {
            var streamified = wrapCallback(val);
            streamified.__HighlandStreamifiedFunction__ = true;
            inp[key + suffix] = streamified;
        }
    }
    return inp;
}
addToplevelMethod('streamifyAll', function (arg) {
    if (typeof arg !== 'function' && typeof arg !== 'object') {
        throw new TypeError('takes an object or a constructor function');
    }
    var suffix = 'Stream';
    var ret = streamifyAll(this.wrapCallback, arg, suffix);
    if (isClass(arg)) {
        ret.prototype = streamifyAll(this.wrapCallback, arg.prototype, suffix);
    }
    return ret;
});
_.add = _.curry(function (a, b) {
    return a + b;
});
_.not = function (x) {
    return !x;
};
});

class ResponseError extends Error {
  constructor(code, message) {
    super(message);
    this.statusCode = code;
  }
}

class RequestStream extends stream.PassThrough {
  constructor(opts) {
    super(opts);
  }
}
var requestStream = ((transport, agent) => (options, buffer) => {
  const requestOptions = Object.assign({}, options, {
    agent
  });
  const s = new RequestStream();
  const req = transport.request(requestOptions, r => {
    r.on('error', handleError);
    if (r.statusCode >= 400) handleError(new ResponseError(r.statusCode), true);
    s.responseHeaders = r.headers;
    s.statusCode = r.statusCode;
    r.pipe(s);
  });
  if (buffer) {
    req.setHeader('Content-Length', buffer.length.toString());
    req.setHeader('Transfer-Encoding', 'identity');
    req.write(buffer);
  }
  s.abort = req.abort.bind(req);
  req.on('error', handleError);
  req.end();
  return s;
  function handleError(err, keepOpen) {
    s.emit('error', err);
    if (!keepOpen) s.end();
  }
});

var errorBuffer = (s => {
  let incomingErr;
  return s.consume((err, chunk, push, next) => {
    if (chunk === lib.nil) return handleNil(chunk, push);else if (err) handleErr(err, push);else handleChunks(chunk, push);
    next();
  });
  function handleErr(err, push) {
    if (!err.statusCode) push(err);else incomingErr = err;
  }
  function handleNil(chunk, push) {
    if (incomingErr) push(incomingErr);
    push(null, chunk);
  }
  function handleChunks(chunk, push) {
    if (incomingErr) incomingErr.message += chunk.toString('utf-8');else push(null, chunk);
  }
});

var addRequestInfo = (options => (error, push) => {
  error.message = `${error.message} From ${options.method} request to ${options.path}`;
  push(error);
});

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var buildOptions = ((_ref) => {
  let {
    method = 'GET',
    rejectUnauthorized = false,
    headers = {}
  } = _ref,
      rest = objectWithoutProperties(_ref, ['method', 'rejectUnauthorized', 'headers']);
  const options = Object.assign({
    method,
    rejectUnauthorized,
    headers: Object.assign({
      Connection: 'keep-alive',
      'Transfer-Encoding': 'chunked'
    }, headers)
  }, rest);
  options.path = options.path.replace(/^\/*/, '/').replace(/\/*$/, '/');
  if (rest.qs) {
    const queryString = querystring.stringify(rest.qs);
    options.path = `${options.path}?${queryString}`;
  }
  return options;
});

var bufferRequest = ((transport, agent) => (options, buffer) => {
  const opts = buildOptions(options || {});
  const resp = {};
  let gotError = false;
  const s = requestStream(transport, agent)(opts, buffer);
  function onFinish(req, callback) {
    function StreamError(err) {
      this.__HighlandStreamError__ = true;
      this.error = err;
    }
    req.on('end', callback).on('close', callback).on('error', err => {
      s2._generator.source.write(new StreamError(err));
    });
    return () => {
      req.removeListener('end', callback);
      req.removeListener('close', callback);
      req.removeListener('error', callback);
    };
  }
  const s2 = lib(s, onFinish);
  const s3 = s2.through(errorBuffer).map(x => x.toString('utf-8')).collect().map(xs => xs.join('')).consume((err, body, push, next) => {
    if (err) {
      gotError = true;
      push(err);
      next();
    } else if (body === lib.nil) {
      if (!gotError) {
        resp.headers = s.responseHeaders;
        resp.statusCode = s.statusCode;
        push(null, resp);
      }
      push(null, lib.nil);
    } else {
      resp.body = body;
      next();
    }
  }).errors(addRequestInfo(opts));
  s3.abort = s.abort;
  return s3;
});

var waitForRequests = (agent => done => {
  const x = setInterval(() => {
    if (Object.keys(agent.sockets).length) return;
    clearInterval(x);
    done();
  }, 10);
});

var ObjProto = Object.prototype;
var isEmpty_1 = isEmpty;
var isArray_1 = Array.isArray || isArray;
var isObject_1 = isObject;
var has_1 = has;
function isEmpty(obj) {
  if (obj == null) return true;
  if (isArray(obj) || typeof obj === 'string') return obj.length === 0;
  for (var key in obj) if (has(obj, key)) return false;
  return true;
}
function isArray(obj) {
  return ObjProto.toString.call(obj) === '[object Array]';
}
function isObject(obj) {
  return typeof obj === 'function' || typeof obj === 'object' && !!obj;
}
function has(obj, key) {
  return ObjProto.hasOwnProperty.call(obj, key);
}
var util = {
	isEmpty: isEmpty_1,
	isArray: isArray_1,
	isObject: isObject_1,
	has: has_1
};

var TERMINALS = { ',': 1, '/': 2, '(': 3, ')': 4 };
var compiler = compile$1;
function compile$1(text) {
  if (!text) return null;
  return parse$1(scan(text));
}
function scan(text) {
  var i = 0;
  var len = text.length;
  var tokens = [];
  var name = '';
  var ch;
  function maybePushName() {
    if (!name) return;
    tokens.push({ tag: '_n', value: name });
    name = '';
  }
  for (; i < len; i++) {
    ch = text.charAt(i);
    if (TERMINALS[ch]) {
      maybePushName();
      tokens.push({ tag: ch });
    } else {
      name += ch;
    }
  }
  maybePushName();
  return tokens;
}
function parse$1(tokens) {
  return _buildTree(tokens, {}, []);
}
function _buildTree(tokens, parent, stack) {
  var props = {};
  var token;
  var peek;
  while (token = tokens.shift()) {
    if (token.tag === '_n') {
      token.type = 'object';
      token.properties = _buildTree(tokens, token, stack);
      peek = stack[stack.length - 1];
      if (peek && peek.tag === '/') {
        stack.pop();
        _addToken(token, props);
        return props;
      }
    } else if (token.tag === ',') {
      return props;
    } else if (token.tag === '(') {
      stack.push(token);
      parent.type = 'array';
      continue;
    } else if (token.tag === ')') {
      stack.pop(token);
      return props;
    } else if (token.tag === '/') {
      stack.push(token);
      continue;
    }
    _addToken(token, props);
  }
  return props;
}
function _addToken(token, props) {
  props[token.value] = { type: token.type };
  if (!util.isEmpty(token.properties)) {
    props[token.value].properties = token.properties;
  }
}

var filter_1 = filter$1;
function filter$1(obj, compiledMask) {
  return util.isArray(obj) ? _arrayProperties(obj, compiledMask) : _properties(obj, compiledMask);
}
function _arrayProperties(arr, mask) {
  var obj = _properties({ _: arr }, { _: {
      type: 'array',
      properties: mask
    } });
  return obj && obj._;
}
function _properties(obj, mask) {
  var maskedObj, key, value, ret, retKey, typeFunc;
  if (!obj || !mask) return obj;
  if (util.isArray(obj)) maskedObj = [];else if (util.isObject(obj)) maskedObj = {};
  for (key in mask) {
    if (!util.has(mask, key)) continue;
    value = mask[key];
    ret = undefined;
    typeFunc = value.type === 'object' ? _object : _array;
    if (key === '*') {
      ret = _forAll(obj, value.properties, typeFunc);
      for (retKey in ret) {
        if (!util.has(ret, retKey)) continue;
        maskedObj[retKey] = ret[retKey];
      }
    } else {
      ret = typeFunc(obj, key, value.properties);
      if (typeof ret !== 'undefined') maskedObj[key] = ret;
    }
  }
  return maskedObj;
}
function _forAll(obj, mask, fn) {
  var ret = {};
  var key;
  var value;
  for (key in obj) {
    if (!util.has(obj, key)) continue;
    value = fn(obj, key, mask);
    if (typeof value !== 'undefined') ret[key] = value;
  }
  return ret;
}
function _object(obj, key, mask) {
  var value = obj[key];
  if (util.isArray(value)) return _array(obj, key, mask);
  return mask ? _properties(value, mask) : value;
}
function _array(object, key, mask) {
  var ret = [];
  var arr = object[key];
  var obj;
  var maskedObj;
  var i;
  var l;
  if (!util.isArray(arr)) return _properties(arr, mask);
  if (util.isEmpty(arr)) return arr;
  for (i = 0, l = arr.length; i < l; i++) {
    obj = arr[i];
    maskedObj = _properties(obj, mask);
    if (typeof maskedObj !== 'undefined') ret.push(maskedObj);
  }
  return ret.length ? ret : undefined;
}

function mask(obj, mask) {
  return filter_1(obj, compiler(mask)) || null;
}
mask.compile = compiler;
mask.filter = filter_1;
var lib$1 = mask;

var jsonMask = (mask => {
  if (!mask) return x => x;
  return x => {
    const response = lib$1(x, mask);
    if (response !== null) return response;
    throw new ResponseError(400,
    `The json mask did not match the response and as a result returned null. Examine the mask: "${mask}"`);
  };
});

var bufferJsonRequest = ((transport, agent) => options => {
  let opts = Object.assign({}, options);
  let mask;
  if (typeof opts.jsonMask === 'string') {
    mask = opts.jsonMask;
    delete opts.jsonMask;
  }
  let buffer;
  if (options.json) {
    buffer = new Buffer(JSON.stringify(options.json));
    opts = Object.assign({}, opts, {
      headers: Object.assign({}, opts.headers || {}, {
        Accept: 'application/json',
        'Content-Type': 'application/json; charset=UTF-8'
      })
    });
    delete opts.json;
  }
  const s = bufferRequest(transport, agent)(opts, buffer);
  const s2 = s.map(x => {
    const masker = jsonMask(mask);
    try {
      const body = masker(JSON.parse(x.body));
      return {
        headers: x.headers,
        statusCode: x.statusCode,
        body
      };
    } catch (e) {
      throw new Error(`Could not parse ${x.body} to JSON.`);
    }
  });
  s2.abort = s.abort;
  return s2;
});

const transports = {
  http,
  https
};
var getReq = ((transport = 'https', agent) => {
  const t = transports[transport];
  agent = agent ||
  new t.Agent({
    keepAlive: true,
    maxSockets: Infinity,
    rejectUnauthorized: false
  });
  return {
    bufferJsonRequest: bufferJsonRequest(t, agent),
    bufferRequest: bufferRequest(t, agent),
    waitForRequests: waitForRequests(agent)
  };
});

var start_pacemaker = "Start Pacemaker on this host.";
var update_devices = "Update device info.";
var volume_status_unconfigured = "This volume cannot be used as a Lustre target until a primary server is selected.";
var found_not_TYPE_for_filesystem = "Found no %ss for filesystem %s";
var ssh_authentication_tooltip = "Choose a method to authenticate on the given hostname / PDSH expression.";
var no_dismiss_message_command = "This command is incomplete. When it has completed you may dismiss.";
var fqdn_matches = "Indicates if there is match between the DNS lookup at the manager server of the user-supplied hostname, and the DNS lookup at the storage server of the self-reported fully-qualified domain name (FQDN) of the storage server.";
var hsm_control_shutdown = "Stop HSM coordinator for this filesystem. No new requests may be scheduled.";
var update_nids = "Update NIDs.";
var remove_file_system = "Remove file system. This file system's contents will remain intact until its volumes are reused in another file system.";
var poweron_host = "Switch on power to this server.";
var unconfigure_corosync = "Unconfiguring Corosync";
var remove_target_XXXX_from_filesystem = "Remove target %s from the filesystem\n*** WARNING *** this is destructive to your filesystem and irreversible";
var add_copytool_host = "The host upon which this copytool instance will be configured to run.";
var resolve = "Indicates if a DNS lookup performed at the manager server, of the fully-qualified domain name (FQDN) of the storage server, succeeded.";
var volume_short = "A LUN or block device used as a metadata or object storage target in a Lustre file system.";
var configure_host_fencing = "Configure fencing agent on this host.";
var configure_target = "Configure target mount points.";
var stop_ost = "Stop the OST. When an OST is stopped, clients are unable to access the files stored on this OST.";
var server_status_configured = "This server has been configured for use with the manager GUI.";
var start_lnet = "Start the LNet networking layer.";
var configure_pacemaker_on = "Configure Pacemaker on %s.";
var deploy_failed_to_register_host = "Failed to register host %s: rc=%s\n'%s'\n'%s'";
var set_host_profile_on = "Setting host profile on %s";
var monitored_filesystem = "This file system is monitored and may not be modified within Integrated Manager for Lustre";
var discovered_filesystem_with_n_MDTs_and_n_OSTs = "Discovered filesystem %s with %s MDTs and %s OSTs";
var conflict_diff = "This row has changed from %(initial)s locally and %(remote)s remotely. Click to set value to %(remote)s";
var stop_pacemaker_on = "Stop Pacemaker on %s.";
var pacemaker_configuration_failed = "Pacemaker configuration failed";
var continue_server_configuration = "Continue Server Configuration";
var start_corosync = "Start Corosync on this host.";
var shutdown_host = "Initiate an orderly shutdown on the host. Any HA-capable targets running on the host will be failed over to a peer. Non-HA-capable targets will be unavailable until the host has been restarted.";
var deploy_agent = "Deploy agent to host.";
var dashboard_filter_fs = "Choose a file system to view. To view a target for a file system, select a file system, and then a target.";
var commands = "Shows past and currently running commands that the manager is executing to perform tasks, such as formatting or starting a file system.";
var remove_copytool = "Deconfigure and remove this copytool from the manager database.";
var enable_corosync = "Enable Corosync on this host.";
var root_password_input_tooltip = "The password of the root user.";
var pdsh_placeholder = "Enter hostname / hostlist expression.";
var server_status_lnet_unloaded = "The LNet kernel module is not currently loaded on this server.";
var server_waiting = "Please wait while the server data is processed.";
var stop_lnet = "Shut down the LNet networking layer and stop any targets running on this server.";
var install_packages_on_host_long = "Install packages on this server";
var lnet_state = "The status of the LNet networking layer on this server.";
var format_target = "Format target.";
var fqdn_resolves = "Indicates if a DNS lookup at the manager server, of the self-reported fully-qualified domain name (FQDN) of the storage server, succeeded.";
var opening_lustre_firewall_port = "Opening Lustre firewall port (988:tcp) on %s";
var configure_corosync_on = "Configure Corosync on %s.";
var reverse_resolve = "Indicates if a DNS lookup by the storage server of the fully-qualified domain name (FQDN) of the manager server succeeded.";
var discovered_no_new_filesystem = "Discovered no new filesystems";
var continue_as_anonymous = "Click this link to continue as an anonymous user. This user has restricted privileges on how they can use the Integrated Manager for Lustre software.";
var type = "The type of storage device.";
var setup_monitored_host = "Setup this monitored host";
var another_key_tooltip = "A private key that is a pair with a public key on this new server, may be used. A passphrase can be supplied for an encrypted private key.";
var status_light = "Indicates current system health. Green: The file system is operating normally.  Yellow: The system may be operating in a degraded mode. Red: This system may be down or is severely degraded. Click to view all system event and alert status messages. Count indicates the number of issues causing the status color.";
var remove_configured_server = "Remove this server. Any file systems or targets that rely on this server will also be removed.";
var existing_keys_tooltip = "If the IML Manager servers are configured with a set of SSH keys, those keys will be used to register the new servers.";
var no_dismiss_message_alert = "This alert relates to an active problem. When the problem is fixed you may dismiss.";
var configure_lnet = "Configure LNet for %s";
var add_copytool_filesystem = "The filesystem for which this copytool will perform HSM actions.";
var configure_copytool = "Configure this copytool on the worker.";
var state_changed = "Time at which the state last changed, either detected or as a result of user action.";
var server_status_lnet_up = "LNet networking is started on this server.";
var configure_ntp = "Configured the NTP Client on the host";
var setup_worker_host = "Setup this worker host";
var deploying_host = "Deploying agent to host %s";
var powercycle_host = "Switch the power to this server off and then back on again. Any HA-capable targets running on the server will be failed over to a peer. Non-HA-capable targets will be unavailable until the server has finished booting.";
var ping = "Indicates if an ICMP ping from the server running Integrated Manager for Lustre, to the server, succeeded.";
var install_packages_on_host = "Install packages on server %s";
var force_remove = "WARNING: This command is destructive. This command should only be performed when the Remove command has been unsuccessful. This command will remove this server from the Integrated Manager for Lustre configuration, but Integrated Manager for Lustre software will not be removed from this server.  All targets that depend on this server will also be removed without any attempt to unconfigure them. To completely remove the Integrated Manager for Lustre software from this server (allowing it to be added to another Lustre file system) you must first contact technical support. You should only perform this command if this server is permanently unavailable, or has never been successfully deployed using Integrated Manager for Lustre software.";
var remove_mgt = "Remove this MGT. The contents will remain intact until the volume is reused for a new target.";
var add_copytool_default_filesystem_option = "Select a filesystem.";
var remove_lustre_client_mount = "Remove this Lustre client entry from the configuration.";
var status = "Indicates the status of high availability (HA) configuration for this volume (ha = available for HA, noha = not configured for HA).";
var local_diff = "This row has changed locally. Click to reset value to %(initial)s";
var access_denied_eula = "To proceed, a superuser must first login and accept the EULA.";
var remove_monitored_configured_server = "Remove this server.";
var unconfigure_ntp = "Unconfigured the NTP Client on the host";
var unconfigure_pacemaker = "Unconfigure Pacemaker on this host.";
var command_detail = "View details about this command.";
var start_ost = "Start the object storage target (OST).";
var discovered_target = "Discovered %s '%s' on server %s";
var root_password_tooltip = "Use standard password based auth, if you don't have SSH keys to use.";
var discovered_no_new_target = "Discovered no new %ss";
var add_target_to_pacemaker_config = "Adding target %s to pacemaker configuration";
var stop_file_system = "Stop the metadata and object storage targets, thus making the file system unavailable to clients.";
var start_copytool = "Start this copytool to begin processing HSM requests.";
var remote_diff = "This row has changed remotely. Click to set value to %(remote)s.";
var copyright_year = "2019";
var stop_mdt = "Stop the MDT. When an MDT is stopped, the file system becomes unavailable until the MDT is started again. If an object reference is known to a client, the client can continue to access the object in the file system after the MDT is shut down, but will not be able to obtain new object references.";
var dashboard_filter_type = "Choose to view servers or file systems.";
var setup_worker_host_on = "Setup worker host %s";
var add_copytool_hsm_arguments = "A vendor-specific list of copytool arguments. Consult your HSM vendor documentation for the correct arguments.";
var start_pacemaker_on = "Start Pacemaker on %s.";
var hsm_control_enabled = "Enable HSM coordinator for this filesystem.";
var start_file_system = "Start the metadata and object storage targets so the file system can be mounted by clients.,";
var unmount_lustre_filesystems = "Unmount all associated Lustre filesystems on this worker.";
var failback_target = "Migrate failed-over target back to primary host";
var auth = "Indicates if the manager server was able to connect, via SSH, to the storage server using the supplied authentication credentials.";
var invoke_agent = "Indicates that the chroma-agent service can be accessed on this server.";
var server_status_lnet_down = "The LNet kernel module is loaded, but LNet networking is not currently started on this server.";
var change_host_profile = "Changing host %s to profile %s";
var load_lnet = "Load the LNet kernel modules.";
var private_key_textarea_tooltip = "The private key for the public key on the servers.";
var configure_pacemaker = "Configure Pacemaker on this host.";
var server_status_unconfigured = "This server has not yet been configured for use with the manager GUI.";
var remove_ost = "Remove the OST from the file system. This OST will no longer be seen in the manager GUI. Caution: When an OST is removed, files stored on the OST will no longer be accessible. To preserve data, manually create a copy of the data elsewhere before removing the OST.";
var migrate_target = "Migrate target.";
var configure_corosync = "Configure Corosync on this host.";
var add_copytool_archive = "The storage backend number. Change this number only if your site policies require multiple storage backends. For more information, consult the \"Lustre Operations Manual\", Section 22.3.1: Archive ID, multiple backends.";
var detect_targets = "Scan for Lustre targets";
var dismiss_message = "Acknowledge this message and move it to the history view.";
var reverse_ping = "Indicates if an ICMP ping from the storage server to manager server succeeded.";
var found_no_primary_mount_point_for_target = "Found no primary mount point for target %s %s";
var add_copytool_default_worker_option = "Select a copytool worker.";
var change_host_state = "Changing host %s to state %s";
var hostname_valid = "Indicates if the self-reported hostname of the storage server is valid (resolves to a non-loopback address).";
var nids = "The Lustre network identifier(s) for the LNet network(s) to which this node belongs.";
var mount_lustre_filesystem = "Mount a Lustre filesystem on this worker.";
var start_mgt = "Start the management target (MGT).";
var deployed_agent_failed_to_contact_manager = "Deployed agent on %s failed to contact manager.";
var mount_lustre_filesystems = "Mount all associated Lustre filesystems on this worker.";
var openssl = "Indicates if OpenSSL is working as expected.";
var volume_long = "Volumes (also called LUNs or block devices) are the underlying units of storage used to create Lustre file systems.  Each Lustre target corresponds to a single volume. If servers in the volume have been configured for high availability, primary and secondary servers can be designated for a Lustre target. Only volumes that are not already in use as Lustre targets or local file systems are shown. A volume may be accessible on one or more servers via different device nodes, and it may be accessible via multiple device nodes on the same host.";
var yum_valid_repos = "Indicates if the storage server's yum configuration has been found to be free of problematic and unsupported software repositories (e.g. EPEL).";
var update_conf_params = "Update Conf params.";
var stop_copytool = "Stop this copytool. In-progress HSM actions will be canceled.";
var goto_dashboard = "Go to the Dashboard";
var inode_size = "Size (in bytes) of the inodes used to store Lustre metadata on the MDS for each file. The default is 512 bytes. In the \"Lustre Operations Manual\", see 5.3.1: Setting the Number of Inodes for the MDS.";
var yum_can_update = "Indicates if the storage server is able to access a repository of packages provided by the Linux distribution vendor. This requirement ensures that yum can satisfy vendor-provided dependencies and obtain vendor-provided software updates.";
var unconfigure_rsyslog = "Unconfigured rsyslog  on the host";
var configure_lnet_not_allowed = "LNet can only be configured in managed mode.";
var stop_corosync = "Stop Corosync on this host.";
var setup_managed_host_on = "Setup managed host %s";
var dashboard_filter_server = "Choose a server to view. To view a target for a server, select a server, and then a target.";
var poweroff_host = "Switch off power to this server. Any HA-capable targets running on the server will be failed over to a peer. Non-HA-capable targets will be unavailable until the server is turned on again.";
var force_remove_copytool = "Remove this copytool from the manager database without any attempt to deconfigure the copytool on the worker node.";
var dashboard_filter_target = "Choose a target to view.";
var register_target = "Register target.";
var unconfigure_pacemaker_on = "Unconfigure Pacemaker on %s";
var unload_lnet = "If LNet is running, stop LNET and unload the LNet kernel module to ensure that it will be reloaded before any targets are started again.";
var advanced_settings = "Use care when changing these parameters as they can significantly impact functionality or performance. For help with these settings, contact your storage solution provider.";
var remove_unconfigured_server = "Remove this unconfigured server.";
var add_copytool_mountpoint = "The filesystem mountpoint on the worker node. Copytool instances require client access to their associated filesystem.";
var add_copytool_bin_path = "The filesystem path to the copytool binary on the worker (e.g. /usr/sbin/lhsmtool_posix).";
var failover_target = "Migrate target to secondary host";
var update_packages = "Update packages.";
var stop_pacemaker = "Stop Pacemaker on this host.";
var remove_mdt = "Remove the MDT from the file system. This MDT will no longer be seen in the manager GUI. Caution: When an MDT is removed, file metadata stored on the MDT will no longer be accessible. To preserve data, manually create a copy of the data elsewhere before removing the MDT.";
var server_waiting_title = "Add Server - Loading";
var install_updates_dialog = "Installs updated software on the selected servers. Select the servers to include in this update operation. Then, click Run to install the updated packages on those servers. Any server that belongs to an active file system is not selectable.";
var setup_managed_host = "Setup this managed host";
var bytes_per_inode = "File system space (in bytes) per MDS inode. The default is 2048, meaning one MDS inode per each 2048 bytes of file system space. In the \"Lustre Operations Manual\", see Section 5.3.3: Setting the Number of Inodes for the MDS.";
var remove_target_from_pacemaker_config = "Removing target %s from pacemaker configuration";
var start_mdt = "Start the metadata target (MDT).";
var setup_monitored_host_on = "Setup monitored host %s";
var managed_filesystem = "This file system is managed by Integrated Manager for Lustre";
var configure_rsyslog = "Configured rsyslog on the host";
var private_key_input_tooltip = "The passphrase for the private key.";
var validating_host = "Validation host configuration %s";
var unmount_lustre_filesystem = "Unmount a Lustre filesystem on this worker.";
var server_waiting_header = "Loading Server Data";
var hsm_control_disabled = "Pause HSM coordinator for this filesystem. New requests will be scheduled when coordinator is enabled.";
var stop_mgt = "Stop the MGT. When an MGT is stopped, clients are unable to make new connections to file systems using the MGT. However, MDT(s) and OST(s) stay up if they have been started and can be stopped and restarted while the MGT is stopped.";
var reboot_host = "Initiate a reboot on the host. Any HA-capable targets running on the host will be failed over to a peer. Non-HA-capable targets will be unavailable until the host has finished rebooting.";
var make_file_system_unavailable = "Make this file system unavailable.";
var user_create_role = "Select your role.";
var user_create_username = "Enter your username.";
var user_create_first_name = "Enter your first name.";
var user_create_last_name = "Enter your last name.";
var user_create_email = "Enter your email address.";
var user_create_password1 = "Enter your password.";
var user_create_password2 = "Confirm your password.";
var helpText = {
	start_pacemaker: start_pacemaker,
	update_devices: update_devices,
	volume_status_unconfigured: volume_status_unconfigured,
	found_not_TYPE_for_filesystem: found_not_TYPE_for_filesystem,
	ssh_authentication_tooltip: ssh_authentication_tooltip,
	no_dismiss_message_command: no_dismiss_message_command,
	fqdn_matches: fqdn_matches,
	hsm_control_shutdown: hsm_control_shutdown,
	update_nids: update_nids,
	remove_file_system: remove_file_system,
	poweron_host: poweron_host,
	unconfigure_corosync: unconfigure_corosync,
	remove_target_XXXX_from_filesystem: remove_target_XXXX_from_filesystem,
	add_copytool_host: add_copytool_host,
	resolve: resolve,
	volume_short: volume_short,
	configure_host_fencing: configure_host_fencing,
	configure_target: configure_target,
	stop_ost: stop_ost,
	server_status_configured: server_status_configured,
	start_lnet: start_lnet,
	configure_pacemaker_on: configure_pacemaker_on,
	deploy_failed_to_register_host: deploy_failed_to_register_host,
	set_host_profile_on: set_host_profile_on,
	monitored_filesystem: monitored_filesystem,
	discovered_filesystem_with_n_MDTs_and_n_OSTs: discovered_filesystem_with_n_MDTs_and_n_OSTs,
	conflict_diff: conflict_diff,
	"volume_status_configured-noha": "This volume is ready to be used as a Lustre target, but is not configured for high availability.",
	stop_pacemaker_on: stop_pacemaker_on,
	pacemaker_configuration_failed: pacemaker_configuration_failed,
	continue_server_configuration: continue_server_configuration,
	start_corosync: start_corosync,
	shutdown_host: shutdown_host,
	deploy_agent: deploy_agent,
	dashboard_filter_fs: dashboard_filter_fs,
	commands: commands,
	remove_copytool: remove_copytool,
	enable_corosync: enable_corosync,
	root_password_input_tooltip: root_password_input_tooltip,
	pdsh_placeholder: pdsh_placeholder,
	server_status_lnet_unloaded: server_status_lnet_unloaded,
	server_waiting: server_waiting,
	"rewrite_target_configuration-tooltip": "Update each target with the current NID for the server with which it is associated.",
	stop_lnet: stop_lnet,
	install_packages_on_host_long: install_packages_on_host_long,
	lnet_state: lnet_state,
	format_target: format_target,
	fqdn_resolves: fqdn_resolves,
	opening_lustre_firewall_port: opening_lustre_firewall_port,
	configure_corosync_on: configure_corosync_on,
	reverse_resolve: reverse_resolve,
	discovered_no_new_filesystem: discovered_no_new_filesystem,
	continue_as_anonymous: continue_as_anonymous,
	type: type,
	"rewrite_target_configuration-dialog": "Select all servers for which the NIDs were re-read by clicking the Rescan NIDs button.  Then click Run to rewrite the Lustre target configuration for targets associated with the selected servers.",
	setup_monitored_host: setup_monitored_host,
	another_key_tooltip: another_key_tooltip,
	status_light: status_light,
	"Start monitoring LNet on %s": "Start monitoring LNet on %s",
	remove_configured_server: remove_configured_server,
	existing_keys_tooltip: existing_keys_tooltip,
	no_dismiss_message_alert: no_dismiss_message_alert,
	configure_lnet: configure_lnet,
	add_copytool_filesystem: add_copytool_filesystem,
	configure_copytool: configure_copytool,
	state_changed: state_changed,
	server_status_lnet_up: server_status_lnet_up,
	configure_ntp: configure_ntp,
	"detect_file_systems-tooltip": "Detect an existing file system to be monitored at the manager GUI.",
	setup_worker_host: setup_worker_host,
	deploying_host: deploying_host,
	powercycle_host: powercycle_host,
	ping: ping,
	install_packages_on_host: install_packages_on_host,
	force_remove: force_remove,
	remove_mgt: remove_mgt,
	add_copytool_default_filesystem_option: add_copytool_default_filesystem_option,
	remove_lustre_client_mount: remove_lustre_client_mount,
	status: status,
	"install_updates_configuration-tooltip": "Install updated software on the selected servers.",
	local_diff: local_diff,
	access_denied_eula: access_denied_eula,
	remove_monitored_configured_server: remove_monitored_configured_server,
	unconfigure_ntp: unconfigure_ntp,
	unconfigure_pacemaker: unconfigure_pacemaker,
	command_detail: command_detail,
	start_ost: start_ost,
	discovered_target: discovered_target,
	root_password_tooltip: root_password_tooltip,
	discovered_no_new_target: discovered_no_new_target,
	add_target_to_pacemaker_config: add_target_to_pacemaker_config,
	stop_file_system: stop_file_system,
	start_copytool: start_copytool,
	remote_diff: remote_diff,
	copyright_year: copyright_year,
	stop_mdt: stop_mdt,
	dashboard_filter_type: dashboard_filter_type,
	setup_worker_host_on: setup_worker_host_on,
	add_copytool_hsm_arguments: add_copytool_hsm_arguments,
	start_pacemaker_on: start_pacemaker_on,
	"volume_status_configured-ha": "This volume is ready to be used for a high-availability (HA) Lustre target.",
	hsm_control_enabled: hsm_control_enabled,
	start_file_system: start_file_system,
	unmount_lustre_filesystems: unmount_lustre_filesystems,
	failback_target: failback_target,
	auth: auth,
	invoke_agent: invoke_agent,
	server_status_lnet_down: server_status_lnet_down,
	change_host_profile: change_host_profile,
	load_lnet: load_lnet,
	private_key_textarea_tooltip: private_key_textarea_tooltip,
	configure_pacemaker: configure_pacemaker,
	server_status_unconfigured: server_status_unconfigured,
	remove_ost: remove_ost,
	migrate_target: migrate_target,
	configure_corosync: configure_corosync,
	"Change lnet state of %s to unconfigured": "Change lnet state of %s to unconfigured",
	add_copytool_archive: add_copytool_archive,
	detect_targets: detect_targets,
	dismiss_message: dismiss_message,
	reverse_ping: reverse_ping,
	found_no_primary_mount_point_for_target: found_no_primary_mount_point_for_target,
	add_copytool_default_worker_option: add_copytool_default_worker_option,
	change_host_state: change_host_state,
	hostname_valid: hostname_valid,
	nids: nids,
	mount_lustre_filesystem: mount_lustre_filesystem,
	start_mgt: start_mgt,
	deployed_agent_failed_to_contact_manager: deployed_agent_failed_to_contact_manager,
	mount_lustre_filesystems: mount_lustre_filesystems,
	openssl: openssl,
	volume_long: volume_long,
	yum_valid_repos: yum_valid_repos,
	update_conf_params: update_conf_params,
	"detect_file_systems-dialog": "Ensure that all storage servers for mounted Lustre targets in the file system to be detected are up and running. Then select the storage servers (including passive failover servers) and click Run to scan them for existing Lustre targets.",
	stop_copytool: stop_copytool,
	goto_dashboard: goto_dashboard,
	inode_size: inode_size,
	yum_can_update: yum_can_update,
	unconfigure_rsyslog: unconfigure_rsyslog,
	configure_lnet_not_allowed: configure_lnet_not_allowed,
	stop_corosync: stop_corosync,
	setup_managed_host_on: setup_managed_host_on,
	dashboard_filter_server: dashboard_filter_server,
	poweroff_host: poweroff_host,
	force_remove_copytool: force_remove_copytool,
	dashboard_filter_target: dashboard_filter_target,
	register_target: register_target,
	unconfigure_pacemaker_on: unconfigure_pacemaker_on,
	unload_lnet: unload_lnet,
	advanced_settings: advanced_settings,
	remove_unconfigured_server: remove_unconfigured_server,
	add_copytool_mountpoint: add_copytool_mountpoint,
	add_copytool_bin_path: add_copytool_bin_path,
	failover_target: failover_target,
	update_packages: update_packages,
	stop_pacemaker: stop_pacemaker,
	remove_mdt: remove_mdt,
	server_waiting_title: server_waiting_title,
	install_updates_dialog: install_updates_dialog,
	setup_managed_host: setup_managed_host,
	bytes_per_inode: bytes_per_inode,
	remove_target_from_pacemaker_config: remove_target_from_pacemaker_config,
	start_mdt: start_mdt,
	setup_monitored_host_on: setup_monitored_host_on,
	managed_filesystem: managed_filesystem,
	configure_rsyslog: configure_rsyslog,
	"Enable LNet on %s": "Enable LNet on %s",
	private_key_input_tooltip: private_key_input_tooltip,
	validating_host: validating_host,
	unmount_lustre_filesystem: unmount_lustre_filesystem,
	"Stop monitoring lnet on %s": "Stop monitoring lnet on %s",
	server_waiting_header: server_waiting_header,
	hsm_control_disabled: hsm_control_disabled,
	stop_mgt: stop_mgt,
	reboot_host: reboot_host,
	make_file_system_unavailable: make_file_system_unavailable,
	user_create_role: user_create_role,
	user_create_username: user_create_username,
	user_create_first_name: user_create_first_name,
	user_create_last_name: user_create_last_name,
	user_create_email: user_create_email,
	user_create_password1: user_create_password1,
	user_create_password2: user_create_password2
};

const env = process.env;
let conf = {
  ALLOW_ANONYMOUS_READ: env.ALLOW_ANONYMOUS_READ === "true",
  BUILD: env.BUILD,
  IS_RELEASE: env.IS_RELEASE,
  NODE_ENV: env.NODE_ENV || "development",
  SERVER_HTTP_URL: env.SERVER_HTTP_URL,
  VERSION: env.VERSION,
  VIEW_SERVER_PORT: env.VIEW_SERVER_PORT
};
if (conf.NODE_ENV === "test") conf = Object.assign({}, conf, {
  ALLOW_ANONYMOUS_READ: true,
  BUILD: "jenkins__",
  SERVER_HTTP_URL: "https://localhost:8000/",
  IS_RELEASE: false,
  VERSION: "",
  VIEW_SERVER_PORT: 8889
});
const parsedServerHttpUrl = url.parse(conf.SERVER_HTTP_URL);
conf = Object.assign({}, conf, {
  API_PORT: parsedServerHttpUrl.port,
  API_URL: url.format(parsedServerHttpUrl),
  HOST_NAME: parsedServerHttpUrl.hostname,
  PARSED_API_URL: parsedServerHttpUrl,
  TEMPLATE_ROOT_NEW: path.sep + path.join("usr", "lib", "iml-manager", "iml-gui") + path.sep,
  TEMPLATE_ROOT_OLD: path.sep + path.join("usr", "lib", "node_modules", "@iml", "old-gui", "templates") + path.sep,
  HELP_TEXT: helpText
});
var conf$1 = conf;

const req = getReq("https");
const serverHttpUrl = url.parse(conf$1.SERVER_HTTP_URL);
var apiRequest = (options => {
  const path$$1 = options.path.replace(/^\/*/, "/").replace(/\/*$/, "/");
  const opts = Object.assign({}, options, {
    localhost: serverHttpUrl.href,
    host: serverHttpUrl.host,
    hostname: serverHttpUrl.hostname,
    port: serverHttpUrl.port,
    path: `/api${path$$1}`
  });
  return req.bufferJsonRequest(opts);
});
const waitForRequests$1 = req.waitForRequests;

var intMap$1 = createCommonjsModule(function (module) {
var hasOwn = Object.prototype.hasOwnProperty;
function IntMap() {
    this.map = {};
    this.size = 0;
}
IntMap.prototype.set = function set(key, value) {
    if (this.has(key)) {
        this.map[key] = value;
    } else {
        this.map[key] = value;
        this.size++;
    }
    return this;
};
IntMap.prototype.get = function get(key) {
    return this.map[key];
};
IntMap.prototype.delete = function intMapDelete(key) {
    var deleted = this.has(key);
    if (deleted) {
        delete this.map[key];
        this.size--;
    }
    return deleted;
};
IntMap.prototype.has = function has(key) {
    return hasOwn.call(this.map, key);
};
IntMap.prototype.forEach = function forEach(f, thisArg) {
    for (var key in this.map) {
        f.call(thisArg, this.map[key], key, this);
    }
};
if (commonjsGlobal.Map === void 0) {
    module.exports = IntMap;
} else {
    module.exports = commonjsGlobal.Map;
}
});

function Queue$1() {
    this._in = [];
    this._out = [];
    this.length = 0;
}
Queue$1.prototype.enqueueAll = function enqueueList(list) {
    this._in = this._in.concat(list);
    this.length += list.length;
};
Queue$1.prototype.enqueue = function enqueue(item) {
    this._in.push(item);
    this.length++;
};
Queue$1.prototype.dequeue = function dequeue() {
    if (this._out.length) {
        this.length--;
        return this._out.pop();
    } else if (this._in.length) {
        this._out = this._in.reverse();
        this._in = [];
        this.length--;
        return this._out.pop();
    } else {
        return undefined;
    }
};
Queue$1.prototype.clear = function clear() {
    this._out = [];
    this._in = [];
    this.length = 0;
};
Queue$1.prototype.toArray = function toArray() {
    var res = [];
    res = res.concat(this._out);
    res.reverse();
    res = res.concat(this._in);
    return res;
};
Queue$1.prototype.toString = function toString() {
    return this.toArray().toString();
};
var queue$1 = Queue$1;

var inherits$1 = require$$0.inherits;
var Readable = stream__default.Readable;
function ReadableProxy(stream$$1, options, nil) {
    Readable.call(this, options);
    this._source = stream$$1;
    this._is_reading = false;
    this._nil = nil;
}
inherits$1(ReadableProxy, Readable);
ReadableProxy.prototype._readOnce = function (state) {
    if (this._is_reading) {
        return;
    }
    var self = this;
    self._is_reading = true;
    self._source.pull(function (err, x) {
        self._is_reading = false;
        if (err) {
            self.emit('error', err);
            self._readOnce(state);
        } else if (x === self._nil) {
            self.push(null);
        } else {
            if (self.push(x)) {
                self._readOnce(state);
            }
        }
    });
};
ReadableProxy.prototype._read = function (ignore) {
    this._readOnce(this._readableState);
};
var readableProxy = ReadableProxy;

var lib$2 = createCommonjsModule(function (module, exports) {
var inherits = require$$0.inherits;
var EventEmitter = events.EventEmitter;
var Decoder = string_decoder.StringDecoder;
var slice = Array.prototype.slice;
var hasOwn = Object.prototype.hasOwnProperty;
var _global = commonjsGlobal;
if (typeof commonjsGlobal !== 'undefined') {
    _global = commonjsGlobal;
} else if (typeof window !== 'undefined') {
    _global = window;
}
var isES5 = function () {
    return Function.prototype.bind && !this;
}();
function bindContext(fn, context) {
    if (isES5) {
        return fn.bind(context);
    } else {
        return function () {
            return fn.apply(context, arguments);
        };
    }
}
var _ = exports = module.exports = __(Stream);
function __(StreamCtor) {
    return function ( xs, secondArg, mappingHint) {
        var s = null;
        if (_.isUndefined(xs)) {
            s = new StreamCtor();
            s.writable = true;
        } else if (_.isStream(xs)) {
            if (!(xs instanceof StreamCtor)) {
                var ret = new StreamCtor();
                xs.on('error', bindContext(ret.write, ret));
                s = xs.pipe(ret);
            } else {
                s = xs;
            }
        } else if (_.isArray(xs)) {
            s = new StreamCtor();
            s._outgoing.enqueueAll(xs);
            s._outgoing.enqueue(_.nil);
        } else if (_.isFunction(xs)) {
            s = new StreamCtor(xs);
        } else if (_.isObject(xs)) {
            if (_.isFunction(xs.on) && _.isFunction(xs.pipe)) {
                var onFinish = _.isFunction(secondArg) ? secondArg : defaultReadableOnFinish;
                s = new StreamCtor();
                s.writable = true;
                pipeReadable(xs, onFinish, s);
                return s.map(function (x) {
                    return x;
                });
            } else if (_.isFunction(xs.then)) {
                s = promiseStream(StreamCtor, xs);
            }
            else if (_.isFunction(xs.next)) {
                    return iteratorStream(StreamCtor, xs);
                } else if (!_.isUndefined(_global.Symbol) && xs[_global.Symbol.iterator]) {
                    return iteratorStream(StreamCtor, xs[_global.Symbol.iterator]());
                } else {
                    throw new Error('Object was not a stream, promise, iterator or iterable: ' + typeof xs);
                }
        } else if (_.isString(xs)) {
            var mapper = hintMapper(mappingHint);
            s = new StreamCtor();
            var callback_func = function () {
                var ctx = mapper.apply(this, arguments);
                s.write(ctx);
            };
            secondArg.on(xs, callback_func);
            var removeMethod = secondArg.removeListener
            || secondArg.unbind;
            if (removeMethod) {
                s._destructors.push(function () {
                    removeMethod.call(secondArg, xs, callback_func);
                });
            }
        } else {
            throw new Error('Unexpected argument type to Stream constructor: ' + typeof xs);
        }
        return s;
    };
}
function checkIsNumber(parameter, paramName) {
    if (typeof parameter != 'number') {
        throw new TypeError('Parameter "' + paramName + '" is not a number.');
    }
}
function checkRange(predicate, message) {
    if (!predicate) {
        throw new RangeError(message);
    }
}
_.isUndefined = function (x) {
    return typeof x === 'undefined';
};
_.isFunction = function (x) {
    return typeof x === 'function';
};
_.isObject = function (x) {
    return typeof x === 'object' && x !== null;
};
_.isString = function (x) {
    return typeof x === 'string';
};
_.isArray = Array.isArray || function (x) {
    return Object.prototype.toString.call(x) === '[object Array]';
};
if (typeof setImmediate === 'undefined') {
    _.setImmediate = function (fn) {
        setTimeout(fn, 0);
    };
} else {
    _.setImmediate = function (fn) {
        setImmediate(fn);
    };
}
if (!_global.nil) {
    _global.nil = {};
}
var nil = _.nil = _global.nil;
_.curry = function (fn ) {
    var args = slice.call(arguments);
    return _.ncurry.apply(this, [fn.length].concat(args));
};
_.ncurry = function (n, fn ) {
    var largs = slice.call(arguments, 2);
    if (largs.length >= n) {
        return fn.apply(this, largs.slice(0, n));
    }
    return _.partial.apply(this, [_.ncurry, n, fn].concat(largs));
};
_.partial = function (f ) {
    var args = slice.call(arguments, 1);
    return function () {
        return f.apply(this, args.concat(slice.call(arguments)));
    };
};
_.flip = _.curry(function (fn, x, y) {
    return fn(y, x);
});
_.compose = function () {
    var fns = slice.call(arguments).reverse();
    return _.seq.apply(null, fns);
};
_.seq = function () {
    var fns = slice.call(arguments);
    return function () {
        if (!fns.length) {
            return null;
        }
        var r = fns[0].apply(this, arguments);
        for (var i = 1; i < fns.length; i++) {
            r = fns[i].call(this, r);
        }
        return r;
    };
};
function nop() {
}
function defaultReadableOnFinish(readable, callback) {
    readable.once('error', callback);
    return function () {
        readable.removeListener('error', callback);
    };
}
function pipeReadable(xs, onFinish, stream$$1) {
    var response = onFinish(xs, streamEndCb);
    var unbound = false;
    var cleanup = null;
    var endOnError = true;
    if (_.isFunction(response)) {
        cleanup = response;
    } else if (response != null) {
        cleanup = response.onDestroy;
        endOnError = !response.continueOnError;
    }
    xs.pipe(stream$$1);
    stream$$1.onDestroy(unbind);
    function streamEndCb(error) {
        if (stream$$1._nil_pushed) {
            return;
        }
        if (error) {
            stream$$1.write(new StreamError(error));
        }
        if (error == null || endOnError) {
            unbind();
            stream$$1.end();
        }
    }
    function unbind() {
        if (unbound) {
            return;
        }
        unbound = true;
        if (cleanup) {
            cleanup();
        }
        if (xs.unpipe) {
            xs.unpipe(stream$$1);
        }
    }
}
function newPullFunction(xs) {
    return function pull(cb) {
        xs.pull(cb);
    };
}
function newDelegateGenerator(pull) {
    return function delegateGenerator(push, next) {
        var self = this;
        pull(function (err, x) {
            var old = self._defer_run_generator;
            self._defer_run_generator = true;
            push(err, x);
            if (x !== nil) {
                next();
            }
            self._defer_run_generator = old;
            if (!old && self._run_generator_deferred) {
                self._runGenerator();
            }
        });
    };
}
function promiseStream(StreamCtor, promise) {
    var nilScheduled = false;
    return new StreamCtor(function (push) {
        promise = promise.then(function (value) {
            nilScheduled = true;
            _.setImmediate(function () {
                push(null, value);
                push(null, nil);
            });
            return null;
        }, function (err) {
            nilScheduled = true;
            _.setImmediate(function () {
                push(err);
                push(null, nil);
            });
            return null;
        });
        if (_.isFunction(promise['finally'])) {
            promise['finally'](function () {
                if (!nilScheduled) {
                    _.setImmediate(function () {
                        push(null, nil);
                    });
                }
                return null;
            });
        }
    });
}
function iteratorStream(StreamCtor, it) {
    return new StreamCtor(function (push, next) {
        var iterElem, iterErr;
        try {
            iterElem = it.next();
        } catch (err) {
            iterErr = err;
        }
        if (iterErr) {
            push(iterErr);
            push(null, _.nil);
        } else if (iterElem.done) {
            if (!_.isUndefined(iterElem.value)) {
                push(null, iterElem.value);
            }
            push(null, _.nil);
        } else {
            push(null, iterElem.value);
            next();
        }
    });
}
function hintMapper(mappingHint) {
    var mappingHintType = typeof mappingHint;
    var mapper;
    if (mappingHintType === 'function') {
        mapper = mappingHint;
    } else if (mappingHintType === 'number') {
        mapper = function () {
            return slice.call(arguments, 0, mappingHint);
        };
    } else if (_.isArray(mappingHint)) {
        mapper = function () {
            var args = arguments;
            return mappingHint.reduce(function (ctx, hint, idx) {
                ctx[hint] = args[idx];
                return ctx;
            }, {});
        };
    } else {
        mapper = function (x) {
            return x;
        };
    }
    return mapper;
}
function pipeStream(src, dest, write, end, passAlongErrors) {
    var resume = null;
    var s = src.consume(function (err, x, push, next) {
        var canContinue;
        if (err) {
            if (passAlongErrors) {
                canContinue = write.call(dest, new StreamError(err));
            } else {
                src.emit('error', err);
                canContinue = true;
            }
        } else if (x === nil) {
            end.call(dest);
            return;
        } else {
            canContinue = write.call(dest, x);
        }
        if (canContinue !== false) {
            next();
        } else {
            resume = next;
        }
    });
    dest.on('drain', onConsumerDrain);
    src.onDestroy(function () {
        dest.removeListener('drain', onConsumerDrain);
    });
    dest.emit('pipe', src);
    s.resume();
    return dest;
    function onConsumerDrain() {
        if (resume) {
            resume();
            resume = null;
        }
    }
}
function Stream(generator) {
    EventEmitter.call(this);
    this.__HighlandStream__ = true;
    this.id = ('' + Math.random()).substr(2, 6);
    this.paused = true;
    this.ended = false;
    this.readable = true;
    this.writable = false;
    this._outgoing = new queue$1();
    this._observers = [];
    this._destructors = [];
    this._send_events = false;
    this._nil_pushed = false;
    this._explicitly_destroyed = false;
    this._request = null;
    this._multiplexer = null;
    this._consumer = null;
    this._generator = generator;
    this._generator_requested = true;
    this._defer_run_generator = false;
    this._run_generator_deferred = false;
    var self = this;
    this._push_fn = function (err, x) {
        if (x === nil) {
            self._run_generator_deferred = false;
        }
        self._writeOutgoing(err ? new StreamError(err) : x);
    };
    this._next_fn = function (xs) {
        if (self._explicitly_destroyed) {
            return;
        }
        if (self._nil_pushed) {
            throw new Error('Cannot call next after nil');
        }
        self._generator_requested = true;
        if (xs) {
            xs = self.create(xs);
            var pull = newPullFunction(xs);
            self._generator = newDelegateGenerator(pull);
        }
        if (!self.paused) {
            if (self._defer_run_generator) {
                self._run_generator_deferred = true;
            } else {
                _.setImmediate(function () {
                    self._runGenerator();
                });
            }
        }
    };
    this.on('newListener', function (ev) {
        if (ev === 'data') {
            self._send_events = true;
            _.setImmediate(bindContext(self.resume, self));
        } else if (ev === 'end') {
            self._send_events = true;
        }
    });
    this.on('removeListener', function (ev) {
        if (ev === 'end' || ev === 'data') {
            var end_listeners = self.listeners('end').length;
            var data_listeners = self.listeners('data').length;
            if (end_listeners + data_listeners === 0) {
                self._send_events = false;
            }
        }
    });
}
inherits(Stream, EventEmitter);
function _addMethod(proto, topLevel) {
    return function (name, f) {
        proto[name] = f;
        var n = f.length;
        function relevel(coerce) {
            return _.ncurry(n + 1, function () {
                var args = slice.call(arguments);
                var s = coerce(args.pop());
                return f.apply(s, args);
            });
        }
        topLevel[name] = relevel(topLevel);
        topLevel[name]._relevel = relevel;
    };
}
var addMethod = _addMethod(Stream.prototype, _);
function _addMethods(proto, topLevel, methods) {
    for (var p in methods) {
        if (hasOwn.call(methods, p)) {
            _addMethod(proto, topLevel)(p, methods[p]);
        }
    }
}
function _addToplevelMethod(topLevel) {
    return function (name, fn) {
        function relevel(_topLevel) {
            var bound = bindContext(fn, _topLevel);
            bound._relevel = relevel;
            return bound;
        }
        topLevel[name] = relevel(topLevel);
    };
}
var addToplevelMethod = _addToplevelMethod(_);
function _addToplevelMethods(topLevel, methods) {
    for (var p in methods) {
        if (hasOwn.call(methods, p)) {
            _addToplevelMethod(topLevel)(p, methods[p]);
        }
    }
}
function use(Super, originalTopLevel) {
    return function (methods, toplevelMethods) {
        function Sub() {
            Stream.apply(this, arguments);
        }
        inherits(Sub, Super);
        function topLevel() {
            return __(Sub).apply(null, arguments);
        }
        for (var p in originalTopLevel) {
            if (hasOwn.call(originalTopLevel, p)) {
                var fn = originalTopLevel[p];
                topLevel[p] = typeof fn._relevel === 'function' ? fn._relevel(topLevel) : fn;
            }
        }
        _addMethods(Sub.prototype, topLevel, methods || {});
        _addToplevelMethods(topLevel, toplevelMethods || {});
        topLevel.use = use(Sub, topLevel);
        return topLevel;
    };
}
_.use = use(Stream, _);
addToplevelMethod('of', function (x) {
    return _([x]);
});
addToplevelMethod('fromError', function (error) {
    return _(function (push) {
        push(error);
        push(null, _.nil);
    });
});
Stream.prototype.create = function () {
    return __(this.constructor).apply(null, arguments);
};
Stream.prototype.createChild = function createChild() {
    var child = this.create.apply(this, arguments);
    child.onDestroy(bindContext(this.destroy, this));
    return child;
};
function StreamError(err) {
    this.__HighlandStreamError__ = true;
    this.error = err;
}
_.isStream = function (x) {
    return _.isObject(x) && !!x.__HighlandStream__;
};
_._isStreamError = function (x) {
    return _.isObject(x) && !!x.__HighlandStreamError__;
};
_._isStreamRedirect = function (x) {
    return _.isObject(x) && !!x.__HighlandStreamRedirect__;
};
Stream.prototype._send = function (token) {
    var err = null,
        x;
    if (_._isStreamError(token)) {
        err = token.error;
        this.readable = false;
    } else {
        x = token;
    }
    if (x === nil) {
        this.readable = false;
    }
    if (this._request) {
        var cb = this._request;
        this._request = null;
        this.pause();
        cb(err, x);
    }
    var observers = this._observers;
    for (var j = 0, len2 = observers.length; j < len2; j++) {
        observers[j].write(token);
    }
    if (this._send_events) {
        if (err) {
            this.emit('error', err);
        } else if (x === nil) {
            this.emit('end');
        } else {
            this.emit('data', x);
        }
    }
    if (x === nil) {
        this._onEnd();
    }
};
Stream.prototype._onEnd = function _onEnd() {
    if (this.ended) {
        return;
    }
    this.pause();
    this.ended = true;
    var i, len, observer;
    for (i = 0, len = this._observers.length; i < len; i++) {
        observer = this._observers[i];
        if (observer.source === this) {
            observer.source = null;
        }
    }
    for (i = 0, len = this._destructors.length; i < len; i++) {
        this._destructors[i].call(this);
    }
    this._outgoing.clear();
    this._observers = [];
    this._destructors = [];
    this._request = null;
    this._multiplexer = null;
    this._consumer = null;
    this._generator = null;
    this._generator_requested = false;
    this._defer_run_generator = false;
    this._run_generator_deferred = false;
};
addMethod('pause', function () {
    this.paused = true;
});
Stream.prototype._emitNext = function (num) {
    var emitted = 0;
    while ((num == null || emitted < num) && this._outgoing.length && !this.paused) {
        this._send(this._outgoing.dequeue());
        emitted++;
    }
};
addMethod('resume', function () {
    if (!this.paused || this._consumer && this._consumer.paused || this._multiplexer && this._multiplexer.paused) {
        return;
    }
    this.paused = false;
    if (this._outgoing.length) {
        this._emitNext();
        if (this.paused) {
            return;
        }
    }
    if (this._generator) {
        this._runGenerator();
    } else {
        this.emit('drain');
    }
});
addMethod('end', function () {
    if (this._nil_pushed) {
        return;
    }
    this.write(nil);
});
addMethod('pipe', function (dest, options) {
    options = options || {};
    var canClose = dest !== process.stdout && dest !== process.stderr && options.end !== false;
    var end;
    if (canClose) {
        end = dest.end;
    } else {
        end = nop;
    }
    return pipeStream(this, dest, dest.write, end, false);
});
addMethod('destroy', function () {
    this._explicitly_destroyed = true;
    if (this.ended) {
        return;
    }
    this.readable = this.writable = false;
    if (!this._nil_pushed) {
        this.end();
    }
    this._onEnd();
});
Stream.prototype._writeOutgoing = function _writeOutgoing(token) {
    if (this._explicitly_destroyed) {
        return;
    }
    if (this._nil_pushed) {
        throw new Error('Cannot write to stream after nil');
    }
    if (token === _.nil) {
        this._nil_pushed = true;
    }
    if (token === nil || _._isStreamError(token)) {
        this.writable = false;
    }
    if (this.paused) {
        this._outgoing.enqueue(token);
    } else {
        this._send(token);
    }
};
Stream.prototype._runGenerator = function () {
    if (!this._generator_requested) {
        return;
    }
    if (this._defer_run_generator) {
        this._run_generator_deferred = true;
        return;
    }
    this._defer_run_generator = true;
    do {
        this._run_generator_deferred = false;
        this._generator_requested = false;
        var gen = this._generator;
        if (gen.__ConsumeGenerator__) {
            var source = gen.source;
            if (source._outgoing.length) {
                var token = source._outgoing.dequeue();
                source._request = gen.pullCb;
                source.paused = false;
                source._send(token);
            } else {
                source._request = gen.pullCb;
                source.resume();
            }
        } else {
            gen.call(this, this._push_fn, this._next_fn);
        }
    } while (this._run_generator_deferred);
    this._defer_run_generator = false;
};
Stream.prototype._removeObserver = function (s) {
    this._observers = this._observers.filter(function (o) {
        return o !== s;
    });
    if (s.source === this) {
        s.source = null;
    }
};
addMethod('consume', function (f) {
    var source = this,
        consumer;
    if (this._consumer) {
        throw new Error('Stream already being consumed, you must either fork() or observe()');
    }
    if (this._multiplexer) {
        throw new Error('Stream has been forked. You must either fork() or observe().');
    }
    var gen = function () {
        source.pull(pullCb);
    };
    gen.__ConsumeGenerator__ = true;
    gen.pullCb = pullCb;
    gen.source = source;
    consumer = this._consumer = this.createChild(gen);
    consumer.id = 'consume:' + consumer.id;
    return consumer;
    function pullCb(err, x) {
        if (consumer._defer_run_generator) {
            f(err, x, consumer._push_fn, consumer._next_fn);
        } else {
            consumer._defer_run_generator = true;
            f(err, x, consumer._push_fn, consumer._next_fn);
            consumer._defer_run_generator = false;
            if (consumer._run_generator_deferred) {
                consumer._runGenerator();
            }
        }
    }
});
addMethod('pull', function (f) {
    if (f == null) {
        throw new Error('Cannot pull() with a null callback.');
    }
    if (this._request) {
        f(new Error('Cannot service a second pull() request while one is in progress.'));
    }
    this._request = f;
    this.resume();
});
addMethod('write', function (x) {
    this._writeOutgoing(x);
    return !this.paused;
});
addMethod('fork', function () {
    if (this._requests) {
        throw new Error('Cannot fork a stream with an outstanding pull() request.');
    }
    if (this._consumer) {
        throw new Error('Cannot fork a stream that has already been consumed().');
    }
    if (!this._multiplexer) {
        this._multiplexer = new StreamMultiplexer(this);
    }
    var s = this._multiplexer.newStream();
    s.id = 'fork:' + s.id;
    return s;
});
addMethod('observe', function () {
    var s = this.create();
    s.id = 'observe:' + s.id;
    s.writable = false;
    var self = this;
    s.onDestroy(function () {
        self._removeObserver(s);
    });
    this._observers.push(s);
    return s;
});
addMethod('errors', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            f(err, push);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            push(null, x);
            next();
        }
    });
});
addMethod('stopOnError', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            f(err, push);
            push(null, nil);
        } else if (x === nil) {
            push(null, nil);
        } else {
            push(null, x);
            next();
        }
    });
});
addMethod('each', function (f) {
    var self = this;
    var s = this.consume(function (err, x, push, next) {
        if (err) {
            self.emit('error', err);
        } else if (x === nil) {
            push(null, nil);
        } else {
            f(x);
            next();
        }
    });
    s.resume();
    return s;
});
addMethod('apply', function (f) {
    return this.toArray(function (args) {
        f.apply(null, args);
    });
});
addMethod('toArray', function (f) {
    var self = this;
    return this.collect().pull(function (err, x) {
        if (err) {
            self.emit('error', err);
        } else {
            f(x);
        }
    });
});
addMethod('done', function (f) {
    if (this.ended) {
        f();
        return null;
    }
    var self = this;
    return this.consume(function (err, x, push, next) {
        if (err) {
            self.emit('error', err);
        } else if (x === nil) {
            f();
        } else {
            next();
        }
    }).resume();
});
Stream.prototype.onDestroy = function onDestroy(f) {
    this._destructors.push(f);
    return this;
};
function toCallbackHandler(transformName, cb) {
    var value;
    var hasValue = false;
    return function (err, x, push, next) {
        if (err) {
            push(null, nil);
            if (hasValue) {
                cb(new Error(transformName + ' called on stream emitting multiple values'));
            } else {
                cb(err);
            }
        } else if (x === nil) {
            if (hasValue) {
                cb(null, value);
            } else {
                cb();
            }
        } else {
            if (hasValue) {
                push(null, nil);
                cb(new Error(transformName + ' called on stream emitting multiple values'));
            } else {
                value = x;
                hasValue = true;
                next();
            }
        }
    };
}
addMethod('toCallback', function (cb) {
    this.consume(toCallbackHandler('toCallback', cb)).resume();
});
addMethod('toPromise', function (PromiseCtor) {
    var self = this;
    return new PromiseCtor(function (resolve, reject) {
        self.consume(toCallbackHandler('toPromise', function (err, res) {
            if (err) {
                reject(err);
            } else {
                resolve(res);
            }
        })).resume();
    });
});
addMethod('toNodeStream', function (options) {
    return new readableProxy(this, options, _.nil);
});
addMethod('map', function (f) {
    if (!_.isFunction(f)) {
        throw new Error('map expects a function as its only argument.');
    }
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var fnVal, fnErr;
            try {
                fnVal = f(x);
            } catch (e) {
                fnErr = e;
            }
            push(fnErr, fnVal);
            next();
        }
    });
});
addMethod('doto', function (f) {
    return this.map(function (x) {
        f(x);
        return x;
    });
});
Stream.prototype.tap = Stream.prototype.doto;
_.tap = _.doto;
addMethod('ratelimit', function (num, ms) {
    if (num < 1) {
        throw new Error('Invalid number of operations per ms: ' + num);
    }
    var sent = 0;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            if (sent < num) {
                sent++;
                push(null, x);
                next();
            } else {
                setTimeout(function () {
                    sent = 1;
                    push(null, x);
                    next();
                }, ms);
            }
        }
    });
});
addMethod('flatMap', function (f) {
    return this.map(f).sequence();
});
addMethod('pluck', function (prop) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else if (_.isObject(x)) {
            push(null, x[prop]);
            next();
        } else {
            push(new Error('Expected Object, got ' + typeof x));
            next();
        }
    });
});
var objectOnly = _.curry(function (strategy, x) {
    if (_.isObject(x)) {
        return strategy(x);
    } else {
        throw new Error('Expected Object, got ' + typeof x);
    }
});
addMethod('pickBy', function (f) {
    return this.map(objectOnly(function (x) {
        var out = {};
        var seen = isES5 ? Object.create(null) : {};
        var obj = x;
        function testAndAdd(prop) {
            if (seen[prop] !== true && f(x[prop], prop)) {
                out[prop] = x[prop];
                seen[prop] = true;
            }
        }
        if (isES5) {
            do {
                Object.getOwnPropertyNames(obj).forEach(testAndAdd);
                obj = Object.getPrototypeOf(obj);
            } while (obj);
        } else {
            for (var k in x) {
                testAndAdd(k);
            }
        }
        return out;
    }));
});
addMethod('pick', function (properties) {
    return this.map(objectOnly(function (x) {
        var out = {};
        for (var i = 0, length = properties.length; i < length; i++) {
            var p = properties[i];
            if (p in x) {
                out[p] = x[p];
            }
        }
        return out;
    }));
});
addMethod('filter', function (f) {
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var fnVal, fnErr;
            try {
                fnVal = f(x);
            } catch (e) {
                fnErr = e;
            }
            if (fnErr) {
                push(fnErr);
            } else if (fnVal) {
                push(null, x);
            }
            next();
        }
    });
});
addMethod('flatFilter', function (f) {
    return this.flatMap(function (x) {
        return f(x).take(1).otherwise(errorStream()).flatMap(function (bool) {
            return _(bool ? [x] : []);
        });
    });
    function errorStream() {
        return _(function (push) {
            push(new Error('Stream returned by function was empty.'));
            push(null, _.nil);
        });
    }
});
addMethod('reject', function (f) {
    return this.filter(_.compose(_.not, f));
});
addMethod('find', function (f) {
    return this.filter(f).take(1);
});
addMethod('findWhere', function (props) {
    return this.where(props).take(1);
});
addMethod('group', function (f) {
    var lambda = _.isString(f) ? _.get(f) : f;
    return this.reduce(function (m, o) {
        var key = lambda(o);
        if (!hasOwn.call(m, key)) {
            m[key] = [];
        }
        m[key].push(o);
        return m;
    }, {});
});
addMethod('compact', function () {
    return this.filter(function (x) {
        return x;
    });
});
addMethod('where', function (props) {
    return this.filter(function (x) {
        for (var k in props) {
            if (x[k] !== props[k]) {
                return false;
            }
        }
        return true;
    });
});
addMethod('uniqBy', function (compare) {
    var uniques = [];
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(err, x);
        } else {
            var seen = false;
            var hasErr;
            for (var i = 0, len = uniques.length; i < len; i++) {
                try {
                    seen = compare(x, uniques[i]);
                } catch (e) {
                    hasErr = e;
                    seen = true;
                }
                if (seen) {
                    break;
                }
            }
            if (!seen) {
                uniques.push(x);
                push(null, x);
            }
            if (hasErr) {
                push(hasErr);
            }
            next();
        }
    });
});
addMethod('uniq', function () {
    if (!_.isUndefined(_global.Set)) {
        var uniques = new _global.Set(),
            size = uniques.size;
        return this.consume(function (err, x, push, next) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(err, x);
            }
            else if (x !== x) {
                    push(null, x);
                    next();
                } else {
                    uniques.add(x);
                    if (uniques.size > size) {
                        size = uniques.size;
                        push(null, x);
                    }
                    next();
                }
        });
    }
    return this.uniqBy(function (a, b) {
        return a === b;
    });
});
addMethod('zipAll', function () {
    var returned = 0;
    var z = [];
    var finished = false;
    function nextValue(index, max, src, push, next) {
        src.pull(function (err, x) {
            if (err) {
                push(err);
                nextValue(index, max, src, push, next);
            } else if (x === _.nil) {
                if (!finished) {
                    finished = true;
                    push(null, nil);
                }
            } else {
                returned++;
                z[index] = x;
                if (returned === max) {
                    push(null, z);
                    next();
                }
            }
        });
    }
    var self = this;
    return self.collect().flatMap(function (array) {
        if (!array.length) {
            return self.create([]);
        }
        return self.create(function (push, next) {
            returned = 0;
            z = [];
            for (var i = 0, length = array.length; i < length; i++) {
                nextValue(i, length, array[i], push, next);
            }
        });
    });
});
addMethod('zipEach', function (ys) {
    return this.create([this]).concat(this.create(ys).map(bindContext(this.create, this))).zipAll();
});
addMethod('zip', function (ys) {
    return this.create([this, this.create(ys)]).zipAll();
});
addMethod('batch', function (n) {
    return this.batchWithTimeOrCount(-1, n);
});
addMethod('batchWithTimeOrCount', function (ms, n) {
    var batched = [],
        timeout;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (batched.length > 0) {
                push(null, batched);
                clearTimeout(timeout);
            }
            push(null, nil);
        } else {
            batched.push(x);
            if (batched.length === n) {
                push(null, batched);
                batched = [];
                clearTimeout(timeout);
            } else if (batched.length === 1 && ms >= 0) {
                timeout = setTimeout(function () {
                    push(null, batched);
                    batched = [];
                }, ms);
            }
            next();
        }
    });
});
addMethod('intersperse', function (separator) {
    var started = false;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else {
            if (started) {
                push(null, separator);
            } else {
                started = true;
            }
            push(null, x);
            next();
        }
    });
});
addMethod('splitBy', function (sep) {
    var decoder = new Decoder();
    var buffer = false;
    function drain(x, push) {
        buffer = (buffer || '') + decoder.write(x);
        var pieces = buffer.split(sep);
        buffer = pieces.pop();
        pieces.forEach(function (piece) {
            push(null, piece);
        });
    }
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (_.isString(buffer)) {
                drain(decoder.end(), push);
                push(null, buffer);
            }
            push(null, nil);
        } else {
            drain(x, push);
            next();
        }
    });
});
addMethod('split', function () {
    return this.splitBy(/\r?\n/);
});
addMethod('slice', function (start, end) {
    if (start == null) {
        start = 0;
    }
    if (end == null) {
        end = Infinity;
    }
    checkIsNumber(start, 'start');
    checkIsNumber(end, 'end');
    checkRange(start >= 0, 'start cannot be negative.');
    checkRange(end >= 0, 'end cannot be negative.');
    var index = 0;
    if (start === 0 && end === Infinity) {
        return this;
    } else if (start >= end) {
        return _([]);
    }
    var s = this.consume(function (err, x, push, next) {
        var done = x === nil;
        if (err) {
            push(err);
        } else if (!done && index++ >= start) {
            push(null, x);
        }
        if (!done && index < end) {
            next();
        } else {
            push(null, nil);
        }
    });
    s.id = 'slice:' + s.id;
    return s;
});
addMethod('take', function (n) {
    var s = this.slice(0, n);
    s.id = 'take:' + s.id;
    return s;
});
addMethod('drop', function (n) {
    return this.slice(n, Infinity);
});
addMethod('head', function () {
    return this.take(1);
});
addMethod('last', function () {
    var nothing = {};
    var prev = nothing;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (prev !== nothing) {
                push(null, prev);
            }
            push(null, nil);
        } else {
            prev = x;
            next();
        }
    });
});
addMethod('sortBy', function (f) {
    return this.collect().invoke('sort', [f]).sequence();
});
addMethod('sort', function () {
    return this.sortBy();
});
addMethod('through', function (target) {
    var output;
    if (_.isFunction(target)) {
        return target(this);
    } else {
        target.pause();
        output = this.createChild();
        this.on('error', writeErr);
        target.on('error', writeErr);
        return this.pipe(target).pipe(output);
    }
    function writeErr(err) {
        output.write(new StreamError(err));
    }
});
addToplevelMethod('pipeline', function () {
    if (!arguments.length) {
        return this();
    }
    var start = arguments[0],
        rest,
        startHighland;
    if (!_.isStream(start) && !_.isFunction(start.resume)) {
        start = this();
        startHighland = start;
        rest = slice.call(arguments);
    } else {
        startHighland = this(start);
        rest = slice.call(arguments, 1);
    }
    var end = rest.reduce(function (src, dest) {
        return src.through(dest);
    }, startHighland);
    var wrapper = this(function (push, next) {
        end.pull(function (err, x) {
            push(err, x);
            if (x !== nil) {
                next();
            }
        });
    });
    wrapper.writable = true;
    wrapper.write = function (x) {
        return start.write(x);
    };
    wrapper.end = function () {
        wrapper.writable = false;
        return start.end();
    };
    start.on('drain', function () {
        wrapper.emit('drain');
    });
    return wrapper;
});
addMethod('sequence', function () {
    var original = this;
    var curr = this;
    return this.create(function (push, next) {
        curr.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (_.isArray(x)) {
                if (onOriginalStream()) {
                    x.forEach(function (y) {
                        push(null, y);
                    });
                } else {
                    push(null, x);
                }
                next();
            } else if (_.isStream(x)) {
                if (onOriginalStream()) {
                    curr = x;
                    next();
                } else {
                    push(null, x);
                    next();
                }
            } else if (x === nil) {
                if (onOriginalStream()) {
                    push(null, nil);
                } else {
                    curr = original;
                    next();
                }
            } else {
                if (onOriginalStream()) {
                    push(new Error('Expected Stream, got ' + typeof x));
                    next();
                } else {
                    push(null, x);
                    next();
                }
            }
        });
    });
    function onOriginalStream() {
        return curr === original;
    }
});
Stream.prototype.series = Stream.prototype.sequence;
_.series = _.sequence;
addMethod('flatten', function () {
    var curr = this;
    var stack = [];
    return this.createChild(function (push, next) {
        curr.pull(function (err, x) {
            if (err) {
                push(err);
                next();
                return;
            }
            if (_.isArray(x)) {
                x = _(x);
            }
            if (_.isStream(x)) {
                stack.push(curr);
                curr = x;
                next();
            } else if (x === nil) {
                if (stack.length) {
                    curr = stack.pop();
                    next();
                } else {
                    push(null, nil);
                }
            } else {
                push(null, x);
                next();
            }
        });
    });
});
addMethod('parallel', function (n) {
    var source = this;
    var running = [];
    var ended = false;
    var reading_source = false;
    if (typeof n !== 'number') {
        throw new Error('Must specify a number to parallel().');
    }
    if (n <= 0) {
        throw new Error('The parallelism factor must be positive');
    }
    return this.create(function (push, next) {
        if (running.length < n && !ended && !reading_source) {
            reading_source = true;
            source.pull(function (err, x) {
                reading_source = false;
                if (err) {
                    push(err);
                } else if (x === nil) {
                    ended = true;
                } else if (!_.isStream(x)) {
                    push(new Error('Expected Stream, got ' + typeof x));
                } else {
                    var run = { stream: x, buffer: [] };
                    running.push(run);
                    x.consume(function (_err, y, _push, _next) {
                        if (running[0] === run) {
                            if (y === nil) {
                                running.shift();
                                flushBuffer();
                                next();
                            } else {
                                push(_err, y);
                            }
                        } else {
                            run.buffer.push([_err, y]);
                        }
                        if (y !== nil) {
                            _next();
                        }
                    }).resume();
                }
                return next();
            });
        } else if (!running.length && ended) {
            push(null, nil);
        }
        function flushBuffer() {
            while (running.length && running[0].buffer.length) {
                var buf = running[0].buffer;
                for (var i = 0; i < buf.length; i++) {
                    if (buf[i][1] === nil) {
                        running.shift();
                        break;
                    } else {
                        push.apply(null, buf[i]);
                    }
                }
                buf.length = 0;
            }
        }
    });
});
addMethod('otherwise', function (ys) {
    var xs = this;
    return xs.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (_.isFunction(ys)) {
                next(ys());
            } else {
                next(ys);
            }
        } else {
            push(null, x);
            next(xs);
        }
    });
});
addMethod('append', function (y) {
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, y);
            push(null, _.nil);
        } else {
            push(err, x);
            next();
        }
    });
});
addMethod('reduce', function (f, z) {
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, z);
            push(null, _.nil);
        } else if (err) {
            push(err);
            next();
        } else {
            try {
                z = f(z, x);
            } catch (e) {
                push(e);
                push(null, _.nil);
                return;
            }
            next();
        }
    });
});
addMethod('reduce1', function (f) {
    var self = this;
    return self.createChild(function (push, next) {
        self.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                next(self.reduce(f, x));
            }
        });
    });
});
addMethod('collect', function () {
    var xs = [];
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, xs);
            push(null, nil);
        } else {
            xs.push(x);
            next();
        }
    });
});
addMethod('scan', function (f, z) {
    var self = this;
    return this.create([z]).concat(self.consume(function (err, x, push, next) {
        if (x === nil) {
            push(null, _.nil);
        } else if (err) {
            push(err);
            next();
        } else {
            try {
                z = f(z, x);
            } catch (e) {
                push(e);
                push(null, _.nil);
                return;
            }
            push(null, z);
            next();
        }
    }));
});
addMethod('scan1', function (f) {
    var self = this;
    return self.createChild(function (push, next) {
        self.pull(function (err, x) {
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                next(self.scan(f, x));
            }
        });
    });
});
function HighlandTransform(push) {
    this.push = push;
}
HighlandTransform.prototype['@@transducer/init'] = function () {
    return this.push;
};
HighlandTransform.prototype['@@transducer/result'] = function (push) {
    return push;
};
HighlandTransform.prototype['@@transducer/step'] = function (push, input) {
    push(null, input);
    return push;
};
addMethod('transduce', function transduce(xf) {
    var transform = null,
        memo = null;
    return this.consume(function (err, x, push, next) {
        if (transform == null) {
            transform = xf(new HighlandTransform(push));
            memo = transform['@@transducer/init']();
        }
        if (err) {
            push(err);
            next();
        } else if (x === _.nil) {
            runResult(push, memo);
        } else {
            var res = runStep(push, memo, x);
            if (!res) {
                return;
            }
            memo = res;
            if (memo['@@transducer/reduced']) {
                runResult(memo['@@transducer/value']);
            } else {
                next();
            }
        }
    });
    function runResult(push, _memo) {
        try {
            transform['@@transducer/result'](_memo);
        } catch (e) {
            push(e);
        }
        push(null, _.nil);
    }
    function runStep(push, _memo, x) {
        try {
            return transform['@@transducer/step'](_memo, x);
        } catch (e) {
            push(e);
            push(null, _.nil);
            return null;
        }
    }
});
addMethod('concat', function (ys) {
    ys = _(ys);
    return this.consume(function (err, x, push, next) {
        if (x === nil) {
            next(ys);
        } else {
            push(err, x);
            next();
        }
    });
});
addMethod('merge', function () {
    var self = this;
    var srcs = [];
    var srcsNeedPull = [],
        first = true,
        async = false;
    return this.create(function (push, next) {
        if (first) {
            first = false;
            getSourcesSync(push, next);
        }
        if (srcs.length === 0) {
            push(null, nil);
        } else if (srcsNeedPull.length) {
            pullFromAllSources(push, next);
            next();
        } else {
            async = true;
        }
    });
    function srcPullHandler(push, next, src) {
        return function (err, x) {
            if (err) {
                push(err);
                srcsNeedPull.push(src);
            } else if (x === nil) {
                srcs = srcs.filter(function (s) {
                    return s !== src;
                });
            } else {
                if (src === self) {
                    srcs.push(x);
                    srcsNeedPull.push(x);
                    srcsNeedPull.unshift(self);
                } else {
                    push(null, x);
                    srcsNeedPull.push(src);
                }
            }
            if (async) {
                async = false;
                next();
            }
        };
    }
    function pullFromAllSources(push, next) {
        var _srcs = srcsNeedPull;
        srcsNeedPull = [];
        _srcs.forEach(function (src) {
            src.pull(srcPullHandler(push, next, src));
        });
    }
    function getSourcesSync(push, next) {
        var asynchronous;
        var done = false;
        var pull_cb = function (err, x) {
            asynchronous = false;
            if (done) {
                srcPullHandler(push, next, self)(err, x);
            } else {
                if (err) {
                    push(err);
                } else if (x === nil) {
                    done = true;
                } else {
                    srcs.push(x);
                    srcsNeedPull.push(x);
                }
            }
        };
        while (!done) {
            asynchronous = true;
            self.pull(pull_cb);
            if (asynchronous) {
                done = true;
                srcs.unshift(self);
            }
        }
    }
});
addMethod('mergeWithLimit', function mergeWithLimit(n) {
    var self = this;
    var processCount = 0;
    var waiting = false;
    if (typeof n !== 'number' || n < 1) {
        throw new Error('mergeWithLimit expects a positive number, but got: ' + n);
    }
    if (n === Infinity) {
        return this.merge();
    }
    return this.create(function (push, next) {
        self.pull(function (err, x) {
            var done = x === nil;
            if (err) {
                push(err);
                next();
            } else if (x === nil) {
                push(null, nil);
            } else {
                processCount++;
                push(err, x);
                x.onDestroy(function () {
                    processCount--;
                    if (waiting) {
                        waiting = false;
                        next();
                    }
                });
                if (!done && processCount < n) {
                    next();
                } else {
                    waiting = true;
                }
            }
        });
    }).merge();
});
addMethod('invoke', function (method, args) {
    return this.map(function (x) {
        return x[method].apply(x, args);
    });
});
addMethod('nfcall', function (args) {
    var wrap = _.wrapCallback._relevel(bindContext(this.create, this));
    return this.map(function (x) {
        return wrap(x).apply(x, args);
    });
});
addMethod('throttle', function (ms) {
    var last = 0 - ms;
    return this.consume(function (err, x, push, next) {
        var now = new Date().getTime();
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            push(null, nil);
        } else if (now - ms >= last) {
            last = now;
            push(null, x);
            next();
        } else {
            next();
        }
    });
});
addMethod('debounce', function (ms) {
    var t = null;
    var nothing = {};
    var last = nothing;
    return this.consume(function (err, x, push, next) {
        if (err) {
            push(err);
            next();
        } else if (x === nil) {
            if (t) {
                clearTimeout(t);
            }
            if (last !== nothing) {
                push(null, last);
            }
            push(null, nil);
        } else {
            last = x;
            if (t) {
                clearTimeout(t);
            }
            t = setTimeout(function () {
                push(null, x);
            }, ms);
            next();
        }
    });
});
addMethod('latest', function () {
    var nothing = {},
        latest = nothing,
        errors = [],
        ended = false,
        onValue = null;
    this.consume(function (err, x, push, next) {
        if (onValue != null) {
            var cb = onValue;
            onValue = null;
            cb(err, x);
        }
        if (err) {
            errors.push(err);
            next();
        } else if (x === nil) {
            ended = true;
        } else {
            latest = x;
            next();
        }
    }).resume();
    return this.create(function (push, next) {
        var oldErrors = errors;
        errors = [];
        if (!oldErrors.length && latest === nothing && !ended) {
            onValue = function (err, x) {
                push(err, x);
                if (x !== nil) {
                    next();
                }
            };
        } else {
            oldErrors.forEach(push);
            if (latest !== nothing) {
                push(null, latest);
            }
            if (ended) {
                push(null, nil);
            } else {
                next();
            }
        }
    });
});
function StreamMultiplexer(stream$$1) {
    this._stream = stream$$1;
    this._consumers = new intMap$1();
    this._requests = new intMap$1();
    this._nonce = 0;
    this._emitting = false;
    this._repeatEmit = false;
    this._cached_value = null;
    this.paused = true;
    var self = this;
    this._pullCb = function _pullCb(err, x) {
        self.paused = true;
        if (self._requests.size === self._consumers.size) {
            self._send(err, x);
        } else {
            self._saved_token = [err, x];
        }
    };
}
StreamMultiplexer.prototype._send = function _send(err, x) {
    var requests = this._requests;
    this._requests = new intMap$1();
    requests.forEach(function (requestCb) {
        requestCb(err, x);
    });
};
StreamMultiplexer.prototype._resume = function _resume() {
    if (this._emitting) {
        this._repeatEmit = true;
        return;
    }
    this._emitting = true;
    do {
        this._repeatEmit = false;
        if (this._requests.size === this._consumers.size) {
            if (this._saved_token) {
                var token = this._saved_token;
                this._saved_token = null;
                this.paused = true;
                this._send(token[0], token[1]);
            } else if (this.paused) {
                this.paused = false;
                this._stream.pull(this._pullCb);
            }
        }
    } while (this._repeatEmit);
    this._emitting = false;
};
StreamMultiplexer.prototype.pull = function pull(id, cb) {
    if (!cb) {
        return;
    }
    if (this._requests.has(id)) {
        cb(new Error('Cannot service a second pull() request while one is in progress.'));
        return;
    }
    this._requests.set(id, cb);
    this._resume();
};
StreamMultiplexer.prototype.newStream = function newStream() {
    var self = this,
        id = self._nonce++;
    self._consumers.set(id, true);
    var generator = newDelegateGenerator(function (cb) {
        self.pull(id, cb);
    });
    return new this._stream.constructor(generator).onDestroy(function () {
        self.removeConsumer(id);
    });
};
StreamMultiplexer.prototype.removeConsumer = function removeConsumer(id) {
    if (!this._consumers.has(id)) {
        return;
    }
    this._consumers.delete(id);
    this._requests.delete(id);
    this._resume();
};
_.values = function (obj) {
    return _.keys(obj).map(function (k) {
        return obj[k];
    });
};
function keys(obj) {
    var keysArray = [];
    for (var k in obj) {
        if (hasOwn.call(obj, k)) {
            keysArray.push(k);
        }
    }
    return keysArray;
}
_.keys = function (obj) {
    return _(keys(obj));
};
_.pairs = function (obj) {
    return _.keys(obj).map(function (k) {
        return [k, obj[k]];
    });
};
_.extend = _.curry(function (extensions, target) {
    for (var k in extensions) {
        if (hasOwn.call(extensions, k)) {
            target[k] = extensions[k];
        }
    }
    return target;
});
_.get = _.curry(function (prop, obj) {
    return obj[prop];
});
_.set = _.curry(function (prop, val, obj) {
    obj[prop] = val;
    return obj;
});
_.log = function () {
    console.log.apply(console, arguments);
};
addToplevelMethod('wrapCallback', function (f, mappingHint) {
    var stream$$1 = this;
    var mapper = hintMapper(mappingHint);
    return function () {
        var self = this;
        var args = slice.call(arguments);
        return stream$$1(function (push) {
            var cb = function (err) {
                if (err) {
                    push(err);
                } else {
                    var cbArgs = slice.call(arguments, 1);
                    var v = mapper.apply(this, cbArgs);
                    push(null, v);
                }
                push(null, nil);
            };
            f.apply(self, args.concat([cb]));
        });
    };
});
addToplevelMethod('wrapAsync', function (f) {
    var stream$$1 = this;
    return function () {
        var promise;
        try {
            promise = f.apply(this, arguments);
            if (!_.isObject(promise) || !_.isFunction(promise.then)) {
                return _.fromError(new Error('Wrapped function did not return a promise'));
            }
            return stream$$1(promise);
        } catch (e) {
            return _.fromError(e);
        }
    };
});
function isClass(fn) {
    if (!(typeof fn === 'function' && fn.prototype)) {
        return false;
    }
    var getKeys = isES5 ? Object.getOwnPropertyNames : keys;
    var allKeys = getKeys(fn.prototype);
    return allKeys.length > 0 && !(allKeys.length === 1 && allKeys[0] === 'constructor');
}
function inheritedKeys(obj) {
    var allProps = {};
    var curr = obj;
    var handleProp = function (prop) {
        allProps[prop] = true;
    };
    while (Object.getPrototypeOf(curr)) {
        var props = Object.getOwnPropertyNames(curr);
        props.forEach(handleProp);
        curr = Object.getPrototypeOf(curr);
    }
    return keys(allProps);
}
function streamifyAll(wrapCallback, inp, suffix) {
    var getKeys = isES5 ? inheritedKeys : keys;
    var allKeys = getKeys(inp);
    for (var i = 0, len = allKeys.length; i < len; i++) {
        var key = allKeys[i];
        var val;
        try {
            val = inp[key];
        } catch (e) {
        }
        if (val && typeof val === 'function' && !isClass(val) && !val.__HighlandStreamifiedFunction__) {
            var streamified = wrapCallback(val);
            streamified.__HighlandStreamifiedFunction__ = true;
            inp[key + suffix] = streamified;
        }
    }
    return inp;
}
addToplevelMethod('streamifyAll', function (arg) {
    if (typeof arg !== 'function' && typeof arg !== 'object') {
        throw new TypeError('takes an object or a constructor function');
    }
    var suffix = 'Stream';
    var ret = streamifyAll(this.wrapCallback, arg, suffix);
    if (isClass(arg)) {
        ret.prototype = streamifyAll(this.wrapCallback, arg.prototype, suffix);
    }
    return ret;
});
_.add = _.curry(function (a, b) {
    return a + b;
});
_.not = function (x) {
    return !x;
};
});

const exec = lib$2.wrapCallback(childProcess.exec);
var getUname = (() => lib$2(["m", "n", "r", "s", "v"]).flatMap(arg => exec(`uname -${arg}`)).map(x => x.toString().trim()).collect().map(zipObject(["sysname", "nodename", "release", "version", "machine"])));

var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;
var _freeGlobal = freeGlobal;

var freeSelf = typeof self == 'object' && self && self.Object === Object && self;
var root = _freeGlobal || freeSelf || Function('return this')();
var _root = root;

var Symbol$1 = _root.Symbol;
var _Symbol = Symbol$1;

var objectProto = Object.prototype;
var hasOwnProperty = objectProto.hasOwnProperty;
var nativeObjectToString = objectProto.toString;
var symToStringTag = _Symbol ? _Symbol.toStringTag : undefined;
function getRawTag(value) {
  var isOwn = hasOwnProperty.call(value, symToStringTag),
      tag = value[symToStringTag];
  try {
    value[symToStringTag] = undefined;
  } catch (e) {}
  var result = nativeObjectToString.call(value);
  {
    if (isOwn) {
      value[symToStringTag] = tag;
    } else {
      delete value[symToStringTag];
    }
  }
  return result;
}
var _getRawTag = getRawTag;

var objectProto$1 = Object.prototype;
var nativeObjectToString$1 = objectProto$1.toString;
function objectToString(value) {
  return nativeObjectToString$1.call(value);
}
var _objectToString = objectToString;

var nullTag = '[object Null]',
    undefinedTag = '[object Undefined]';
var symToStringTag$1 = _Symbol ? _Symbol.toStringTag : undefined;
function baseGetTag(value) {
    if (value == null) {
        return value === undefined ? undefinedTag : nullTag;
    }
    return symToStringTag$1 && symToStringTag$1 in Object(value) ? _getRawTag(value) : _objectToString(value);
}
var _baseGetTag = baseGetTag;

function isObject$1(value) {
  var type = typeof value;
  return value != null && (type == 'object' || type == 'function');
}
var isObject_1$1 = isObject$1;

var asyncTag = '[object AsyncFunction]',
    funcTag = '[object Function]',
    genTag = '[object GeneratorFunction]',
    proxyTag = '[object Proxy]';
function isFunction(value) {
    if (!isObject_1$1(value)) {
        return false;
    }
    var tag = _baseGetTag(value);
    return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
}
var isFunction_1 = isFunction;

var coreJsData = _root['__core-js_shared__'];
var _coreJsData = coreJsData;

var maskSrcKey = function () {
  var uid = /[^.]+$/.exec(_coreJsData && _coreJsData.keys && _coreJsData.keys.IE_PROTO || '');
  return uid ? 'Symbol(src)_1.' + uid : '';
}();
function isMasked(func) {
  return !!maskSrcKey && maskSrcKey in func;
}
var _isMasked = isMasked;

var funcProto = Function.prototype;
var funcToString = funcProto.toString;
function toSource(func) {
  if (func != null) {
    try {
      return funcToString.call(func);
    } catch (e) {}
    try {
      return func + '';
    } catch (e) {}
  }
  return '';
}
var _toSource = toSource;

var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;
var reIsHostCtor = /^\[object .+?Constructor\]$/;
var funcProto$1 = Function.prototype,
    objectProto$2 = Object.prototype;
var funcToString$1 = funcProto$1.toString;
var hasOwnProperty$1 = objectProto$2.hasOwnProperty;
var reIsNative = RegExp('^' + funcToString$1.call(hasOwnProperty$1).replace(reRegExpChar, '\\$&').replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');
function baseIsNative(value) {
  if (!isObject_1$1(value) || _isMasked(value)) {
    return false;
  }
  var pattern = isFunction_1(value) ? reIsNative : reIsHostCtor;
  return pattern.test(_toSource(value));
}
var _baseIsNative = baseIsNative;

function getValue$1(object, key) {
  return object == null ? undefined : object[key];
}
var _getValue = getValue$1;

function getNative(object, key) {
  var value = _getValue(object, key);
  return _baseIsNative(value) ? value : undefined;
}
var _getNative = getNative;

var defineProperty$1 = function () {
  try {
    var func = _getNative(Object, 'defineProperty');
    func({}, '', {});
    return func;
  } catch (e) {}
}();
var _defineProperty = defineProperty$1;

function baseAssignValue(object, key, value) {
  if (key == '__proto__' && _defineProperty) {
    _defineProperty(object, key, {
      'configurable': true,
      'enumerable': true,
      'value': value,
      'writable': true
    });
  } else {
    object[key] = value;
  }
}
var _baseAssignValue = baseAssignValue;

function eq$1(value, other) {
  return value === other || value !== value && other !== other;
}
var eq_1 = eq$1;

var objectProto$3 = Object.prototype;
var hasOwnProperty$2 = objectProto$3.hasOwnProperty;
function assignValue(object, key, value) {
  var objValue = object[key];
  if (!(hasOwnProperty$2.call(object, key) && eq_1(objValue, value)) || value === undefined && !(key in object)) {
    _baseAssignValue(object, key, value);
  }
}
var _assignValue = assignValue;

function copyObject(source, props, object, customizer) {
  var isNew = !object;
  object || (object = {});
  var index = -1,
      length = props.length;
  while (++index < length) {
    var key = props[index];
    var newValue = customizer ? customizer(object[key], source[key], key, object, source) : undefined;
    if (newValue === undefined) {
      newValue = source[key];
    }
    if (isNew) {
      _baseAssignValue(object, key, newValue);
    } else {
      _assignValue(object, key, newValue);
    }
  }
  return object;
}
var _copyObject = copyObject;

function identity$1(value) {
  return value;
}
var identity_1 = identity$1;

function apply(func, thisArg, args) {
  switch (args.length) {
    case 0:
      return func.call(thisArg);
    case 1:
      return func.call(thisArg, args[0]);
    case 2:
      return func.call(thisArg, args[0], args[1]);
    case 3:
      return func.call(thisArg, args[0], args[1], args[2]);
  }
  return func.apply(thisArg, args);
}
var _apply = apply;

var nativeMax = Math.max;
function overRest(func, start, transform) {
  start = nativeMax(start === undefined ? func.length - 1 : start, 0);
  return function () {
    var args = arguments,
        index = -1,
        length = nativeMax(args.length - start, 0),
        array = Array(length);
    while (++index < length) {
      array[index] = args[start + index];
    }
    index = -1;
    var otherArgs = Array(start + 1);
    while (++index < start) {
      otherArgs[index] = args[index];
    }
    otherArgs[start] = transform(array);
    return _apply(func, this, otherArgs);
  };
}
var _overRest = overRest;

function constant(value) {
  return function () {
    return value;
  };
}
var constant_1 = constant;

var baseSetToString = !_defineProperty ? identity_1 : function (func, string) {
  return _defineProperty(func, 'toString', {
    'configurable': true,
    'enumerable': false,
    'value': constant_1(string),
    'writable': true
  });
};
var _baseSetToString = baseSetToString;

var HOT_COUNT = 800,
    HOT_SPAN = 16;
var nativeNow = Date.now;
function shortOut(func) {
  var count = 0,
      lastCalled = 0;
  return function () {
    var stamp = nativeNow(),
        remaining = HOT_SPAN - (stamp - lastCalled);
    lastCalled = stamp;
    if (remaining > 0) {
      if (++count >= HOT_COUNT) {
        return arguments[0];
      }
    } else {
      count = 0;
    }
    return func.apply(undefined, arguments);
  };
}
var _shortOut = shortOut;

var setToString = _shortOut(_baseSetToString);
var _setToString = setToString;

function baseRest(func, start) {
  return _setToString(_overRest(func, start, identity_1), func + '');
}
var _baseRest = baseRest;

var MAX_SAFE_INTEGER = 9007199254740991;
function isLength(value) {
  return typeof value == 'number' && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
}
var isLength_1 = isLength;

function isArrayLike(value) {
  return value != null && isLength_1(value.length) && !isFunction_1(value);
}
var isArrayLike_1 = isArrayLike;

var MAX_SAFE_INTEGER$1 = 9007199254740991;
var reIsUint = /^(?:0|[1-9]\d*)$/;
function isIndex(value, length) {
  var type = typeof value;
  length = length == null ? MAX_SAFE_INTEGER$1 : length;
  return !!length && (type == 'number' || type != 'symbol' && reIsUint.test(value)) && value > -1 && value % 1 == 0 && value < length;
}
var _isIndex = isIndex;

function isIterateeCall(value, index, object) {
  if (!isObject_1$1(object)) {
    return false;
  }
  var type = typeof index;
  if (type == 'number' ? isArrayLike_1(object) && _isIndex(index, object.length) : type == 'string' && index in object) {
    return eq_1(object[index], value);
  }
  return false;
}
var _isIterateeCall = isIterateeCall;

function createAssigner(assigner) {
  return _baseRest(function (object, sources) {
    var index = -1,
        length = sources.length,
        customizer = length > 1 ? sources[length - 1] : undefined,
        guard = length > 2 ? sources[2] : undefined;
    customizer = assigner.length > 3 && typeof customizer == 'function' ? (length--, customizer) : undefined;
    if (guard && _isIterateeCall(sources[0], sources[1], guard)) {
      customizer = length < 3 ? undefined : customizer;
      length = 1;
    }
    object = Object(object);
    while (++index < length) {
      var source = sources[index];
      if (source) {
        assigner(object, source, index, customizer);
      }
    }
    return object;
  });
}
var _createAssigner = createAssigner;

function baseTimes(n, iteratee) {
  var index = -1,
      result = Array(n);
  while (++index < n) {
    result[index] = iteratee(index);
  }
  return result;
}
var _baseTimes = baseTimes;

function isObjectLike(value) {
  return value != null && typeof value == 'object';
}
var isObjectLike_1 = isObjectLike;

var argsTag = '[object Arguments]';
function baseIsArguments(value) {
  return isObjectLike_1(value) && _baseGetTag(value) == argsTag;
}
var _baseIsArguments = baseIsArguments;

var objectProto$4 = Object.prototype;
var hasOwnProperty$3 = objectProto$4.hasOwnProperty;
var propertyIsEnumerable = objectProto$4.propertyIsEnumerable;
var isArguments = _baseIsArguments(function () {
    return arguments;
}()) ? _baseIsArguments : function (value) {
    return isObjectLike_1(value) && hasOwnProperty$3.call(value, 'callee') && !propertyIsEnumerable.call(value, 'callee');
};
var isArguments_1 = isArguments;

var isArray$1 = Array.isArray;
var isArray_1$1 = isArray$1;

function stubFalse() {
  return false;
}
var stubFalse_1 = stubFalse;

var isBuffer_1 = createCommonjsModule(function (module, exports) {
var freeExports = exports && !exports.nodeType && exports;
var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;
var moduleExports = freeModule && freeModule.exports === freeExports;
var Buffer = moduleExports ? _root.Buffer : undefined;
var nativeIsBuffer = Buffer ? Buffer.isBuffer : undefined;
var isBuffer = nativeIsBuffer || stubFalse_1;
module.exports = isBuffer;
});

var argsTag$1 = '[object Arguments]',
    arrayTag = '[object Array]',
    boolTag = '[object Boolean]',
    dateTag = '[object Date]',
    errorTag = '[object Error]',
    funcTag$1 = '[object Function]',
    mapTag = '[object Map]',
    numberTag = '[object Number]',
    objectTag = '[object Object]',
    regexpTag = '[object RegExp]',
    setTag = '[object Set]',
    stringTag = '[object String]',
    weakMapTag = '[object WeakMap]';
var arrayBufferTag = '[object ArrayBuffer]',
    dataViewTag = '[object DataView]',
    float32Tag = '[object Float32Array]',
    float64Tag = '[object Float64Array]',
    int8Tag = '[object Int8Array]',
    int16Tag = '[object Int16Array]',
    int32Tag = '[object Int32Array]',
    uint8Tag = '[object Uint8Array]',
    uint8ClampedTag = '[object Uint8ClampedArray]',
    uint16Tag = '[object Uint16Array]',
    uint32Tag = '[object Uint32Array]';
var typedArrayTags = {};
typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = true;
typedArrayTags[argsTag$1] = typedArrayTags[arrayTag] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dataViewTag] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag$1] = typedArrayTags[mapTag] = typedArrayTags[numberTag] = typedArrayTags[objectTag] = typedArrayTags[regexpTag] = typedArrayTags[setTag] = typedArrayTags[stringTag] = typedArrayTags[weakMapTag] = false;
function baseIsTypedArray(value) {
    return isObjectLike_1(value) && isLength_1(value.length) && !!typedArrayTags[_baseGetTag(value)];
}
var _baseIsTypedArray = baseIsTypedArray;

function baseUnary(func) {
  return function (value) {
    return func(value);
  };
}
var _baseUnary = baseUnary;

var _nodeUtil = createCommonjsModule(function (module, exports) {
var freeExports = exports && !exports.nodeType && exports;
var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;
var moduleExports = freeModule && freeModule.exports === freeExports;
var freeProcess = moduleExports && _freeGlobal.process;
var nodeUtil = function () {
  try {
    var types = freeModule && freeModule.require && freeModule.require('util').types;
    if (types) {
      return types;
    }
    return freeProcess && freeProcess.binding && freeProcess.binding('util');
  } catch (e) {}
}();
module.exports = nodeUtil;
});

var nodeIsTypedArray = _nodeUtil && _nodeUtil.isTypedArray;
var isTypedArray = nodeIsTypedArray ? _baseUnary(nodeIsTypedArray) : _baseIsTypedArray;
var isTypedArray_1 = isTypedArray;

var objectProto$5 = Object.prototype;
var hasOwnProperty$4 = objectProto$5.hasOwnProperty;
function arrayLikeKeys(value, inherited) {
  var isArr = isArray_1$1(value),
      isArg = !isArr && isArguments_1(value),
      isBuff = !isArr && !isArg && isBuffer_1(value),
      isType = !isArr && !isArg && !isBuff && isTypedArray_1(value),
      skipIndexes = isArr || isArg || isBuff || isType,
      result = skipIndexes ? _baseTimes(value.length, String) : [],
      length = result.length;
  for (var key in value) {
    if ((inherited || hasOwnProperty$4.call(value, key)) && !(skipIndexes && (
    key == 'length' ||
    isBuff && (key == 'offset' || key == 'parent') ||
    isType && (key == 'buffer' || key == 'byteLength' || key == 'byteOffset') ||
    _isIndex(key, length)))) {
      result.push(key);
    }
  }
  return result;
}
var _arrayLikeKeys = arrayLikeKeys;

var objectProto$6 = Object.prototype;
function isPrototype(value) {
  var Ctor = value && value.constructor,
      proto = typeof Ctor == 'function' && Ctor.prototype || objectProto$6;
  return value === proto;
}
var _isPrototype = isPrototype;

function nativeKeysIn(object) {
  var result = [];
  if (object != null) {
    for (var key in Object(object)) {
      result.push(key);
    }
  }
  return result;
}
var _nativeKeysIn = nativeKeysIn;

var objectProto$7 = Object.prototype;
var hasOwnProperty$5 = objectProto$7.hasOwnProperty;
function baseKeysIn(object) {
  if (!isObject_1$1(object)) {
    return _nativeKeysIn(object);
  }
  var isProto = _isPrototype(object),
      result = [];
  for (var key in object) {
    if (!(key == 'constructor' && (isProto || !hasOwnProperty$5.call(object, key)))) {
      result.push(key);
    }
  }
  return result;
}
var _baseKeysIn = baseKeysIn;

function keysIn(object) {
  return isArrayLike_1(object) ? _arrayLikeKeys(object, true) : _baseKeysIn(object);
}
var keysIn_1 = keysIn;

var assignInWith = _createAssigner(function (object, source, srcIndex, customizer) {
  _copyObject(source, keysIn_1(source), object, customizer);
});
var assignInWith_1 = assignInWith;

function overArg(func, transform) {
  return function (arg) {
    return func(transform(arg));
  };
}
var _overArg = overArg;

var getPrototype = _overArg(Object.getPrototypeOf, Object);
var _getPrototype = getPrototype;

var objectTag$1 = '[object Object]';
var funcProto$2 = Function.prototype,
    objectProto$8 = Object.prototype;
var funcToString$2 = funcProto$2.toString;
var hasOwnProperty$6 = objectProto$8.hasOwnProperty;
var objectCtorString = funcToString$2.call(Object);
function isPlainObject(value) {
  if (!isObjectLike_1(value) || _baseGetTag(value) != objectTag$1) {
    return false;
  }
  var proto = _getPrototype(value);
  if (proto === null) {
    return true;
  }
  var Ctor = hasOwnProperty$6.call(proto, 'constructor') && proto.constructor;
  return typeof Ctor == 'function' && Ctor instanceof Ctor && funcToString$2.call(Ctor) == objectCtorString;
}
var isPlainObject_1 = isPlainObject;

var domExcTag = '[object DOMException]',
    errorTag$1 = '[object Error]';
function isError(value) {
    if (!isObjectLike_1(value)) {
        return false;
    }
    var tag = _baseGetTag(value);
    return tag == errorTag$1 || tag == domExcTag || typeof value.message == 'string' && typeof value.name == 'string' && !isPlainObject_1(value);
}
var isError_1 = isError;

var attempt = _baseRest(function (func, args) {
  try {
    return _apply(func, undefined, args);
  } catch (e) {
    return isError_1(e) ? e : new Error(e);
  }
});
var attempt_1 = attempt;

function arrayMap(array, iteratee) {
  var index = -1,
      length = array == null ? 0 : array.length,
      result = Array(length);
  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}
var _arrayMap = arrayMap;

function baseValues(object, props) {
  return _arrayMap(props, function (key) {
    return object[key];
  });
}
var _baseValues = baseValues;

var objectProto$9 = Object.prototype;
var hasOwnProperty$7 = objectProto$9.hasOwnProperty;
function customDefaultsAssignIn(objValue, srcValue, key, object) {
  if (objValue === undefined || eq_1(objValue, objectProto$9[key]) && !hasOwnProperty$7.call(object, key)) {
    return srcValue;
  }
  return objValue;
}
var _customDefaultsAssignIn = customDefaultsAssignIn;

var stringEscapes = {
  '\\': '\\',
  "'": "'",
  '\n': 'n',
  '\r': 'r',
  '\u2028': 'u2028',
  '\u2029': 'u2029'
};
function escapeStringChar(chr) {
  return '\\' + stringEscapes[chr];
}
var _escapeStringChar = escapeStringChar;

var nativeKeys = _overArg(Object.keys, Object);
var _nativeKeys = nativeKeys;

var objectProto$a = Object.prototype;
var hasOwnProperty$8 = objectProto$a.hasOwnProperty;
function baseKeys(object) {
  if (!_isPrototype(object)) {
    return _nativeKeys(object);
  }
  var result = [];
  for (var key in Object(object)) {
    if (hasOwnProperty$8.call(object, key) && key != 'constructor') {
      result.push(key);
    }
  }
  return result;
}
var _baseKeys = baseKeys;

function keys(object) {
  return isArrayLike_1(object) ? _arrayLikeKeys(object) : _baseKeys(object);
}
var keys_1 = keys;

var reInterpolate = /<%=([\s\S]+?)%>/g;
var _reInterpolate = reInterpolate;

function basePropertyOf(object) {
  return function (key) {
    return object == null ? undefined : object[key];
  };
}
var _basePropertyOf = basePropertyOf;

var htmlEscapes = {
  '&': '&amp;',
  '<': '&lt;',
  '>': '&gt;',
  '"': '&quot;',
  "'": '&#39;'
};
var escapeHtmlChar = _basePropertyOf(htmlEscapes);
var _escapeHtmlChar = escapeHtmlChar;

var symbolTag = '[object Symbol]';
function isSymbol(value) {
    return typeof value == 'symbol' || isObjectLike_1(value) && _baseGetTag(value) == symbolTag;
}
var isSymbol_1 = isSymbol;

var INFINITY = 1 / 0;
var symbolProto = _Symbol ? _Symbol.prototype : undefined,
    symbolToString = symbolProto ? symbolProto.toString : undefined;
function baseToString(value) {
  if (typeof value == 'string') {
    return value;
  }
  if (isArray_1$1(value)) {
    return _arrayMap(value, baseToString) + '';
  }
  if (isSymbol_1(value)) {
    return symbolToString ? symbolToString.call(value) : '';
  }
  var result = value + '';
  return result == '0' && 1 / value == -INFINITY ? '-0' : result;
}
var _baseToString = baseToString;

function toString(value) {
  return value == null ? '' : _baseToString(value);
}
var toString_1 = toString;

var reUnescapedHtml = /[&<>"']/g,
    reHasUnescapedHtml = RegExp(reUnescapedHtml.source);
function escape(string) {
    string = toString_1(string);
    return string && reHasUnescapedHtml.test(string) ? string.replace(reUnescapedHtml, _escapeHtmlChar) : string;
}
var _escape = escape;

var reEscape = /<%-([\s\S]+?)%>/g;
var _reEscape = reEscape;

var reEvaluate = /<%([\s\S]+?)%>/g;
var _reEvaluate = reEvaluate;

var templateSettings = {
  'escape': _reEscape,
  'evaluate': _reEvaluate,
  'interpolate': _reInterpolate,
  'variable': '',
  'imports': {
    '_': { 'escape': _escape }
  }
};
var templateSettings_1 = templateSettings;

var reEmptyStringLeading = /\b__p \+= '';/g,
    reEmptyStringMiddle = /\b(__p \+=) '' \+/g,
    reEmptyStringTrailing = /(__e\(.*?\)|\b__t\)) \+\n'';/g;
var reEsTemplate = /\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g;
var reNoMatch = /($^)/;
var reUnescapedString = /['\n\r\u2028\u2029\\]/g;
function template(string, options, guard) {
  var settings = templateSettings_1.imports._.templateSettings || templateSettings_1;
  if (guard && _isIterateeCall(string, options, guard)) {
    options = undefined;
  }
  string = toString_1(string);
  options = assignInWith_1({}, options, settings, _customDefaultsAssignIn);
  var imports = assignInWith_1({}, options.imports, settings.imports, _customDefaultsAssignIn),
      importsKeys = keys_1(imports),
      importsValues = _baseValues(imports, importsKeys);
  var isEscaping,
      isEvaluating,
      index = 0,
      interpolate = options.interpolate || reNoMatch,
      source = "__p += '";
  var reDelimiters = RegExp((options.escape || reNoMatch).source + '|' + interpolate.source + '|' + (interpolate === _reInterpolate ? reEsTemplate : reNoMatch).source + '|' + (options.evaluate || reNoMatch).source + '|$', 'g');
  var sourceURL = 'sourceURL' in options ? '//# sourceURL=' + options.sourceURL + '\n' : '';
  string.replace(reDelimiters, function (match, escapeValue, interpolateValue, esTemplateValue, evaluateValue, offset) {
    interpolateValue || (interpolateValue = esTemplateValue);
    source += string.slice(index, offset).replace(reUnescapedString, _escapeStringChar);
    if (escapeValue) {
      isEscaping = true;
      source += "' +\n__e(" + escapeValue + ") +\n'";
    }
    if (evaluateValue) {
      isEvaluating = true;
      source += "';\n" + evaluateValue + ";\n__p += '";
    }
    if (interpolateValue) {
      source += "' +\n((__t = (" + interpolateValue + ")) == null ? '' : __t) +\n'";
    }
    index = offset + match.length;
    return match;
  });
  source += "';\n";
  var variable = options.variable;
  if (!variable) {
    source = 'with (obj) {\n' + source + '\n}\n';
  }
  source = (isEvaluating ? source.replace(reEmptyStringLeading, '') : source).replace(reEmptyStringMiddle, '$1').replace(reEmptyStringTrailing, '$1;');
  source = 'function(' + (variable || 'obj') + ') {\n' + (variable ? '' : 'obj || (obj = {});\n') + "var __t, __p = ''" + (isEscaping ? ', __e = _.escape' : '') + (isEvaluating ? ', __j = Array.prototype.join;\n' + "function print() { __p += __j.call(arguments, '') }\n" : ';\n') + source + 'return __p\n}';
  var result = attempt_1(function () {
    return Function(importsKeys, sourceURL + 'return ' + source).apply(undefined, importsValues);
  });
  result.source = source;
  if (isError_1(result)) {
    throw result;
  }
  return result;
}
var template_1 = template;

const reduce$1 = function reduce(accum, fn, obj) {
  return Object.keys(obj).reduce((out, key) => fn(obj[key], key, out), accum());
};

function getDirTreeSync(dir, transformPath, dirTree = {}) {
  return fs.readdirSync(dir).reduce((obj, file) => {
    const filePath = path.join(dir, file);
    const s = fs.statSync(filePath);
    if (s.isFile() && /\.html$/.test(filePath)) obj[transformPath(filePath)] = fs.readFileSync(filePath, "utf8");
    if (s.isDirectory()) getDirTreeSync(`${filePath}/`, transformPath, obj);
    return obj;
  }, dirTree);
}

const transformPath = root => p => p.replace(root, "");
const templates = Object.assign({}, getDirTreeSync(conf$1.TEMPLATE_ROOT_OLD, transformPath(conf$1.TEMPLATE_ROOT_OLD)), getDirTreeSync(conf$1.TEMPLATE_ROOT_NEW, transformPath(conf$1.TEMPLATE_ROOT_NEW)));
templateSettings_1.imports = Object.assign({}, templateSettings_1.imports, {
  t(name, data) {
    return templateMap[name](data);
  },
  conf: conf$1,
  getServerDate() {
    return new Date();
  }
});
templateSettings_1.interpolate = /<\$=([\s\S]+?)\$>/g;
templateSettings_1.escape = /<\$-([\s\S]+?)\$>/g;
templateSettings_1.evaluate = /<\$([\s\S]+?)\$>/g;
const templateMap = reduce$1(() => ({}), (val, key, out) => {
  out[key] = template_1(val);
  return out;
}, templates);

const backendErrorTemplate = templateMap["backend_error.html"];
var renderRequestError = ((res, descriptionFn) => err => {
  const description = descriptionFn(err);
  getUname().each(map => {
    const rendered = backendErrorTemplate({
      description,
      debug_info: map
    });
    res.clientRes.end(rendered);
  });
});

var getSession = ((req, res, next) => {
  const cookie = req.clientReq.headers.cookie || "";
  apiRequest({
    path: "/session",
    headers: {
      cookie
    }
  }).stopOnError(renderRequestError(res, err => `Exception rendering resources: ${err.stack}`)).each(response => {
    res.clientRes.setHeader("Set-Cookie", response.headers["set-cookie"]);
    const data = {
      session: response.body,
      cache: {},
      cacheCookie: response.headers["set-cookie"].map(x => (x.match(/((?:csrftoken|sessionid)=[^;]+;)/) || [""])[0]).join(" ")
    };
    next(req, res, data);
  });
});

var getCache = ((req, res, data, next) => {
  let cache;
  const calls = [{ path: "/filesystem", qs: {} }, { path: "/target", qs: {} }, { path: "/host", qs: {} }, { path: "/power_control_type", qs: {} }, { path: "/server_profile", qs: {} }, {
    path: "/lnet_configuration",
    qs: {
      dehydrate__host: false
    }
  }, {
    path: "/alert",
    jsonMask: "objects(affected,message)",
    qs: {
      active: true
    }
  }, {
    path: "/job",
    jsonMask: "objects(write_locks,read_locks,description)",
    qs: {
      state__in: ["pending", "tasked"]
    }
  }];
  if (data.session.user != null || conf$1.ALLOW_ANONYMOUS_READ) cache = lib$2(calls).map((_ref) => {
    let { qs = {} } = _ref,
        rest = objectWithoutProperties(_ref, ["qs"]);
    return apiRequest(Object.assign({
      headers: {
        Cookie: data.cacheCookie
      },
      qs: Object.assign({
        limit: 0
      }, qs)
    }, rest));
  }).parallel(calls.length).pluck("body").pluck("objects");else cache = lib$2(times(always([]))(calls.length));
  cache.collect().map(zipObject(calls.map(call => call.path.slice(1)))).stopOnError(renderRequestError(res, err => `Exception rendering resources: ${err.stack}`)).each(cache => {
    cache.session = data.session;
    data.cache = cache;
    next(req, res, data);
  });
});

var viewRouter = getRouter().addStart(getSession).addStart(getCache);

const indexTemplate = templateMap["index.html"];
var loginRoute = (() => {
  viewRouter.route("/ui/login").get((req, res, data, next) => {
    const session = data.cache.session;
    if (session.user) return res.redirect("/ui/");
    next(req, res, data.cache);
  }).get((req, res, cache, next) => {
    res.clientRes.setHeader("Content-Type", "text/html; charset=utf-8");
    res.clientRes.statusCode = 200;
    const rendered = indexTemplate({
      title: "Login",
      cache: cache
    });
    res.clientRes.end(rendered);
    next(req, res);
  });
});

var cspPolicy = "default-src 'none';\
 child-src 'self';\
 script-src 'self' 'unsafe-inline' 'unsafe-eval';\
 connect-src 'self' wss:;\
 img-src 'self' data:;\
 font-src 'self';\
 style-src 'self' 'unsafe-inline';";

const handler = template => (req, res, data, next) => {
  const session = data.cache.session;
  if (!session.user && !conf$1.ALLOW_ANONYMOUS_READ) return res.redirect("/ui/login/");
  res.clientRes.setHeader("Content-Type", "text/html; charset=utf-8");
  res.clientRes.setHeader("Content-Security-Policy", cspPolicy);
  res.clientRes.statusCode = 200;
  const rendered = template({
    title: "",
    cache: data.cache
  });
  res.clientRes.end(rendered);
  next(req, res);
};
const newTemplate = templateMap["index.html"];
const oldTemplate = templateMap["base.html"];
var indexHandlers = {
  oldHandler: handler(oldTemplate),
  newHandler: handler(newTemplate)
};

var groups = {
  SUPERUSERS: "superusers",
  FS_ADMINS: "filesystem_administrators",
  FS_USERS: "filesystem_users"
};

var groupAllowed = ((groupName, session = { user: { groups: [] } }) => session.user.groups.some(group => {
  if (group.name === groups.SUPERUSERS) return true;
  if (group.name === groups.FS_ADMINS && groupName === groups.FS_USERS) return true;
  return group.name === groupName;
}));

const allowGroup = groupName => (req, res, data, next) => {
  if (!groupAllowed(groupName, data.cache.session)) return res.redirect("/ui/");
  next(req, res, data);
};
const transform = text => text.toLowerCase().split("_").reduce((str, part) => str += capitalize(part));
const capitalize = x => x.charAt(0).toUpperCase() + x.slice(1);
const keys$1 = [];
const vals = [];
Object.keys(groups).forEach(key => {
  keys$1.push(transform(key));
  vals.push(allowGroup(groups[key]));
});
var checkGroup = zipObject(keys$1)(vals);

var indexRoute = (() => {
  viewRouter.route("/ui/configureold/:subpath+").get(checkGroup.fsAdmins).get(indexHandlers.oldHandler);
  viewRouter.route("/ui/targetold/:id").get(checkGroup.fsAdmins).get(indexHandlers.oldHandler);
  viewRouter.route("/ui/storage_resourceold/:id").get(checkGroup.fsAdmins).get(indexHandlers.oldHandler);
  viewRouter.route("/ui/userold/:id").get(checkGroup.fsUsers).get(indexHandlers.oldHandler);
  viewRouter.route("/ui/system_statusold").get(checkGroup.fsAdmins).get(indexHandlers.oldHandler);
  viewRouter.get("/(.*)", indexHandlers.newHandler);
});

https.globalAgent.maxSockets = http.globalAgent.maxSockets = Infinity;
loginRoute();
indexRoute();
var start = (() => {
  const server = http.createServer((req, res) => {
    viewRouter.go(req.url, {
      verb: req.method,
      clientReq: req
    }, {
      clientRes: res,
      redirect(path$$1) {
        res.setHeader("Content-Security-Policy", cspPolicy);
        res.writeHead(302, { Location: path$$1 });
        res.end();
      }
    });
  }).listen(conf$1.VIEW_SERVER_PORT);
  return function stop(done = noop) {
    server.on("close", err => {
      if (err) throw err;
      waitForRequests$1(done);
    });
  };
});

start();
//# sourceMappingURL=bundle.js.map
